#.  Word Memory
#.  a beginner's reading app
#.
#. Copyright (c) 2020 Roland Frank (E-mail: rfrank@deltadatentechnik.de)
#. AndroWish Demo App
#. Tk gui : andra megawidget
#. central labels show discriminating letters in alphabetic order, they are arranged columnwise using the pack geometry manager
#. label and font sizes adjust to the available space
#. layout adjusts to screen orientation 
#. the megawidget displays (unicode) strings, uses TTS and provides a simple select and query mode.
#. 
#.  wordlists from //translate.google.com

if {[catch {package require Tk}]} {
	load /usr/lib/x86_64l-linux-gnu/libtk8.6.so
}
wm attributes . -fullscreen 1
#. configure -bg {}
wm withdraw .
package require math
package require math::interpolate
puts [info procs math::interpolate::*]
package require textutil
package require md5
package require http
#.package require tls
proc check_utf_max {} {
	#. start of supplementary multilingual plane
	set n \U10000
	set i 65533
	scan $n %c i
	if {$i == 65533} {
		error TCL_UTF_MAX
	}
}
if {[catch check_utf_max]} {
	puts "This demo requires AndroWish, if necessary undroidwish or a tclkit with TCL_UTF_MAX=6 - . Only AndroWish has the required fonts and TTS included -"
	exit
}

set k [toplevel .k -width 200 -height 600]
set android [expr {[catch {package require Borg}]==0}]

proc appexit {keycode} {
	#. just has to exist to make Androwish hideable
	#. $toplevel <KeyPress> {appexit %K}
	#. <Control-BackSpace> does not work on Android
	#. will print "Break" 
	#. puts appexit:$k
	if {$keycode == 270} {
		borg withdraw
	}
}
proc log {args} {
	puts "log:<$args>"
}
if {$android} {
	#. next line defunct

	#. works once?
	#. bind .k <Control-BackSpace> {}
	bind .k <Key> {appexit %k}

	set adb_debug 0
	if {$adb_debug} {
		if {[catch {package require tkconclient} err]} {
		} else {
			proc sbin {} {
				#. avoid crash
				#. to be used before parray ::pnr
				fconfigure [set ::tkconclient::socket] -translation binary
			}
			tkconclient::start 4488
			vwait vv
			sbin
		}
	}
} else {
	#. androwish commands not present
	proc borg args {
		puts "dummy borg:$args"
	}
	proc speak {args} {
		puts "speak $args"
	}
	proc sdltk args {
		puts "dummy sdltk:$args"
	}
}
proc shuffle9 {list} {
	foreach a $list {
		if {[string length $a]} {
			set arr($a) 1
		}
	}
	return [array names arr]
}
oo::class create andra {
	variable vkb_width vkb_height 
	variable size_current
	variable bc_valid
	variable dchar
	variable hashme
	variable uca
	variable R
	variable parentframe
	#. pack options
	variable lr rl an ran 
	#. widgets
	variable tau
	variable ltx
	variable delta prefix_frame vkb
	variable input men
	variable suffix aftersuffix
	variable header logo tail
	variable bsf bs
	#. bools
	variable ltr selection portrait portrait_old
	variable istouch
	#. lists
	variable breadcrumbs
	#. strings
	variable mlang
	variable current_suffix
	#. index int
	variable bcidx
	#.
	variable iso_symbol
	#. fonts 
	#. arrays
	variable pnr mod yy force_i vis_hash
	variable odds
	variable ftsi
	variable mod
	#. font names
	variable dfont
	variable picfont
	variable topfont

	variable tf
	#. colors
	variable color_back_bg color_fw_bg color_d_bg color_home_bg uni_fg
	#. traces
	#. kwiz-related
	variable kwizact
	variable kwizlist
	variable kwiz_visited
	variable kwizinfo
	variable kwiztotal kwizcount kwizskipped
	variable visited pics
	#.
	variable render_arascr
	variable adado

	constructor {theframe} {
		variable first_time 1
		set adado 2
		set aftersuffix ""
		set render_arascr 0
		set portrait_old 2
		set size_current 0
		set pics [list]
		set R $theframe
		append R .
		set R [regsub {^\.\.} $R {}]
		set R [regsub {\.$} $R {}]
		set R [regsub {^.$} $R {}]
		#. /assets/app/main.tcl
		set istouch [string equal $::argv0 /assets/app/main.tcl]




		set bcidx 0

		#. avoid this (pack propagation): wm geometry $frame 300x700+0+0



		#.set iso_symbol \U1f527   ;#wrench
		set iso_symbol \U1f530   ;#beginner

		set color_home_bg \#300b
		set color_back_bg \#f9ecee
		set color_d_bg white
		#. set color_fw_bg \#c0eeff
		set color_fw_bg \#c9f9f2



		#
		#. //rkuon.ch //hsluv.org

		set color_home_bg \#f9f3f6

		set color_fw_bg \#e5f9f7
		set color_d_bg \#dcf8c7



		set color_back_bg \#f5f9bf

		set uni_fg black
		set ltx [label $R.ltx -text hi]
		pack $ltx -side top -fill x

		set hh [winfo height $theframe]
		set hh [expr {$hh*2/3}]
		frame $R.f -bg {} -width [winfo width $theframe] -height $hh

		#. when backgroud is set to empty, it will not be redrawn when 
		#. contained frame $vkb is temporarily "pack forget"

		set tau [frame $R.f.tau -bg orange]

		set vkb [frame $R.f.tau.vkb -bg {}]

		set dfont $tau

		set size [expr {-[winfo width $R]/10}]
puts "initial font size: $size"
		font create $dfont -family {Droid Sans} -slant italic -size $size
		set picfont $tau.p

		font create $picfont -family {Droid Sans} -size $size
		set topfont top.f
		append topfont $R

		font create $topfont -family {Droid Sans} -weight bold -size 20
		$ltx configure -font $topfont


		set header [frame $R.f.h -bg brown -borderwidth 2]
		set logo [frame $header.l -bg {}]

		set delta [label $logo.d -fg $uni_fg -bg \#ddf9b0 -text $color_home_bg -justify center -height 2]

		if {[catch {
			set tmp [split $::env(LANG) .]
			set mlang [lindex $tmp 0]
		}]} {
			set mlang de
		}

		set bsf [frame $header.bsf -bg $color_back_bg]


		set bs [label $bsf.bs -text \u300a -width 2 -bg $color_back_bg -fg $uni_fg -justify center]


		#. reconfigure due to ltr
		if {[catch {
		my direction $mlang
		my geometry
		} err]} {puts err=<$err>}

		pack $logo $delta -anchor w -side $lr -fill y

		set men [frame $header.left -borderwidth 1px -relief flat -highlightthickness 1px -highlightbackground yellow -bg white]

		set tail [frame $men.t -bg white]

		set suffix [label $tail.s -text {} -bg white]
			set aftersuffix $suffix
			append aftersuffix _as

			button $aftersuffix -text {x} -bg white -fg blue -command [list [self] backstep]
		set prefix_frame [frame $men.px -bg white -width 1]


		set iwi 5

		set input [entry $tail.i -width 2 -justify center -bg \#fff0ff -borderwidth 0 -relief flat -insertwidth $iwi -insertborderwidth -1 -insertbackground white -insertofftime 1400 -highlightthickness 0 -relief flat]

		#
		if {0} {
			pack $prefix_frame -side $lr -fill y -expand 1 -anchor n$an

			pack $input -side $lr -anchor $an -padx 1 -fill y -expand 1
			pack $suffix -side $lr -anchor $an -fill x
			pack $tail -side $lr
			pack $men -side $lr -fill both -expand 1 -anchor nw
		} else {
			my show_controls
		}
		
		#. uses input + men
		my entry_bindings 

		bind $delta <Button-1> [list [self] home]
		bind $bs <Button-1> [list [self] backstep]
		bind $bsf <Button-1> [list [self] backstep]
		pack $header -side top -padx {4 4} -fill x -expand 1 -anchor n


#. moved to ada


		pack $bs -side $rl -anchor $ran
		pack $bsf -side $lr -fill both -anchor $an -expand 1

		
		#.yyy



		#. expand causes column overflow on bottom
		#. resize to use remaining window area

		set breadcrumbs {}
		set lang de
		if {[catch {set lang $::env(LANG)}]} {
			set lang en
		} else {
			set lang [regsub {[_\.].*$} $lang {}]
		}

		set elements [my mylang $lang]


		set tmp [my force_initials elements]

		if {[string equal $lang $mlang]} {
			$ltx configure -text $lang
		} else {
			$ltx configure -text $lang-$mlang
		}
		#. build loopup array yy
		#. load dependencies

		my load_proc

		my kwiz_init

		set validcount [llength $tmp]

		set fname taupu_
		append fname _$lang



		cd ~
		set dd [pwd]
		set fname [file join $dd $fname]

		if {[catch {
			set fh [open $fname.txt w]
		} err]} {puts err=$err}

		set prev ""
		set line_prev ""
		set skip 0

		foreach line [lsort -unique [my no_explode $tmp]] {

			if {[catch {set item $pnr($line)} err]} {

				set item ""
			} else {
			}

			if {![string equal $item $prev]} {
				#. smallest item at index 0 is written
				puts $fh $line
			} else {

				if {![string length [gcp $line $line_prev]]} {
					puts $fh $line
				} else {
					incr skip
				}
			}
			set prev $item
			set line_prev $line
		}

		seek $fh -1 end
		close $fh

		if {[catch {
		sorted_file2tx $fname.txt
	} err]} {puts err=$err}


		set fh [open dchar.txt w]
		#. to change global dchar
		puts $fh [array get dchar]
		close $fh


		#. needed
puts "@#. needed"

		set fh [open $fname.tx r]
		catch {array unset yy}
		array set yy [read $fh]
		close $fh



		#. after is required else no object is returned
		pack $vkb -side $lr -fill both -expand 1
		pack $tau -side top -fill both -expand 1
		pack $R.f -fill both -expand 1
		my tfon ini
puts "end constructor"
	}
	method kwiz_init {} {

		if {$istouch} {
			bind $ltx <Button-1> [list [self] pkwiz]
		} else {
			bind $ltx <Button-1> [list [self] pkwiz]
		}

		set kwizact ""
		set kwizinfo ""
		set kwizlist [list]
		set kwiz_visited [list]
		set kwiztotal 0
		set kwizcount 0
		set kwizskipped 0

	}
	method load_proc {} {
		set dd [file dirname [info script]]
		source [file join $dd qq/qq.tcl]
		#. get render_arabic
		source [file join $dd ra.tcl]
	}
	method size_default {} {

		set w [winfo width $R]
		#. chars per line
		set chars_per_line [expr {$portrait ? 23. : 40.}]
		set hsize [expr {-int($w/$chars_per_line)}]

		return $hsize
	}
	method geometry {} {
		#. sets value of portrait
		set w [winfo width $R]
		set h [winfo height $R]
		set portrait [expr {$w<$h}]
		set change [expr {$portrait != $portrait_old}]
		if {$change} {
			set portrait_old $portrait


			if {[winfo exists $ltx]} {
				$ltx configure -height [expr {$portrait ? 2 : 1}]
				if {$portrait} {
					$ltx configure -wraplength 0

					$ltx configure -bg \#f9f0c6
				}
			}
			#. commenting next line causes duplicate shifted windows in opposite orientations
			#. still relevant
			update idletasks
		}

		return $change
	}
	method direction {lang} {
		set ltr [expr {!([lsearch {ar fa iw ur ps yi} $lang]>-1)}] 
		if {$ltr} {
			font configure $dfont -slant italic
			set lr left
			set rl right
			set an w
			set ran e
			$delta configure -text $mlang\u300b -bg $color_home_bg
			$bs configure -text \u300a
		} else {
			font configure $dfont -slant roman
			set lr right
			set rl left
			set an e
			set ran w
			$delta configure -text \u300a$mlang -bg $color_home_bg
			$bs configure -text \u300b
		}
	}
	method tree {x} {
		puts "children of $x"
		foreach n [winfo children $x] {
			puts "$n [pack info $n]"
		}
	}
	method var {varname} {
		if {[catch {set res [set $varname]} err]} {
			puts $err
			return
		}
		return $res

	}
	method set {name value} {
		set $name $value
	}
	method entry_bindings {} {

		bind $input <Any-Key> [list [self] newchar %A %k]
		bind $input <Key-Delete> [list [self] newchar "" %k]
		bind $input <Key-BackSpace>  [list [self] backstep]
		bind $input <Key-Home> [list [self] home]
		bind $input <FocusIn> [list $men configure -highlightcolor hotpink]
		bind $input <FocusOut> [list $men configure -border 1px]
		#. bindtags $input SdlTkNoTextInput

	}

	method newchar {singlechar {keycode 0} {recursive 0}} {
puts "newchar:start, singlechar=$singlechar, recursive=$recursive"

		#. important! remove previous input
		if {$keycode != 0} {
			$input delete 0 end
		}
		my hide_suffix
		set fin 0
		if {[string length $singlechar] == 1} {
			if {[catch {set word $vis_hash($singlechar)}]} {
				if {!$recursive} {

					$input configure -fg red
					set toggled [string toupper $singlechar]
					if {[string equal $toggled $singlechar]} {
						set toggled [string tolower $singlechar]
					}

					if {![string equal $singlechar $toggled]} {
						if {[catch {set word $vis_hash($toggled)}]} {

							$input configure -selectforeground red -selectbackground \#aff


							return
						} else {


							#. below will not work here
							#. but in after idle append_suffix



							#. my newchar $toggled $keycode 1
							#. Error: invoked "break" outside of a loop
							after idle [list [self] newchar $toggled $keycode 1]

						}
					
					}
				}
				return
			} else {
				if {$recursive} {
					$input insert end $singlechar
				}
				$input configure -fg black
				#. valid single character
				#. puts newchar:word=<$word>
				set key [my current_prefix]
				if {![string length $key]} {
					pack forget $prefix_frame
				} else {
					if {![winfo ismapped $prefix_frame]} {
						#. fill y for multiline suffix

						pack $prefix_frame -side $lr -anchor n -fill y -expand 1
					}
				}
				set x ""
				foreach {fin x} $word break
				puts "fin=$fin, x=$x"
				if {[string length $x]} {
					set prefix [join $breadcrumbs {}]
					if {[string length $kwizact]} {
						#. refuse invalid path
						set nok [my check_prefix $x]

						if {$nok} {
							#.puts "path refused"
							my show_controls

							return
						}
					}
					if {$fin} {
						my append_suffix $fin $x
					} else {
						lappend breadcrumbs $x
						my bc_row $prefix_frame $x
						my tfon cc
						after idle [list $input delete 0 end]
						#. will not happen: $input delete 0 end

					}
					$input configure -fg black
				} else {
					$input configure -fg red
				}
			}
		} else {
puts "keycode=$keycode"
			if {$keycode == 40} {
				#. key <return>
				if {$bc_valid} {
					my append_suffix 1 ""
				}
			} elseif {$keycode==43} {
				#. tab
				focus $aftersuffix
			} elseif {$keycode==76} {
				#. delete
				my hide_suffix
			} elseif {$keycode==270} {
				borg withdraw
				return
			}
			if {[info exists ::suffix] && [winfo exists $::suffix]} {
				$suffix configure -text {}
				$input delete 0 end
			}
		}
		focus $input




	}
	method current_prefix {{x {}}} {
		return [join $breadcrumbs {}]
	}
	method hide_suffix {} {
		puts hide_suffix:start
		if {[info exists current_suffix] && [winfo exists $suffix]} {
			puts "$suffix set empty"
			$suffix configure -text {}
		}
		if {[info exists aftersuffix]} {
			puts "$aftersuffix forget"
			pack forget $aftersuffix
		}
		puts hide_suffix:end
	}
	method clear {parent} {
		#. purpose: find reason for gui-freeze
		#. before destroy {*}[winfo children PARENT]
		#. seems better
		#. idea to delay row events
		#. after 700 [list bind   ->nok
		#. after idle [....] ok but window may be destroyed meanwhile
		#. todo collect elements with frameclick in list_forward
		#. lappend list_forward [list $w $fin $pele]
		#.      collect elements with backstep   in list_back
		#. lappend list_back $w
		#. -> bind after nada_mem
		foreach x [winfo children $parent] {
			set evbi [bind $x]
			#. remove binding
			foreach eb $evbi {

				bind $x $eb {}

			}
			my clear $x

			destroy $x
		}
	}

	method home {} {
		variable first_time
		set first_time 1

		set breadcrumbs [list]
		my hide_suffix
		my clear $prefix_frame
		set bcidx 0
		$input delete 0 end
		$suffix configure -text ""
		pack forget $aftersuffix
		my tfon CHAR
	}
	method hide_controls {} {
		focus $R
		pack forget $header
	}
	method show_controls {} {
		puts "show_controls:start"

		pack forget $header $logo $delta $men $tail $suffix $prefix_frame $input $bsf $bs
		pack $logo $delta -anchor $an -side $lr -fill y
		pack $prefix_frame -side $lr

		pack $input -side $lr -anchor $an -padx 1 -fill y
		pack $bs -side $rl -anchor $ran

		pack $suffix -side $lr -anchor $an -fill x
		pack $tail -side $lr
		pack $men -side $lr -fill both -expand 1 -anchor nw
		pack $bsf -side $rl -fill both -anchor $an -expand 1



		focus $input

		puts "show_controls:end"
	}
	method backstep {} {
		variable first_time
		puts backstep:start
		$suffix configure -text ""
		if {[winfo exists $aftersuffix]} {
			pack forget $aftersuffix
		}
		if {$bcidx >1} {
			set breadcrumbs [lrange $breadcrumbs 0 end-1]

			destroy {*}[lrange [winfo children $prefix_frame] end-1 end]

	puts "backstep: after cancel"
			$input delete 0 end
			incr bcidx -2
	puts "backstep:bcidx=$bcidx"
			puts "backstep:tfon"

			after idle [list [self] tfon CHAR]
			puts "backstep:after tfon"
		} 
		if {$bcidx==0} {
			set first_time 1
		}
		puts backstep:end
	}
	method tfon {{T {}}} {
puts "tfon:start:$T"
		variable cwid {}

		set rotation [my geometry]

		set pics {}
		set prefix [my current_prefix]
puts "prefix=<$prefix>"
		#.puts X0
		set key $portrait,.f,$prefix
		#.puts X1

		set keep_first [expr {![string length $prefix]}]
		set prefix_len [string length $prefix]
		if {$prefix_len==0} {
			set lp ""
		} elseif {$prefix_len<2} {
			set lp " "
		} else {
			set lp ""
		}

		if {!$portrait} {
			append lp " "
			append lp $prefix
		} elseif {![regexp { $} $prefix]} {
			append lp [regsub {^.*\s} $prefix ~]
		} elseif {[string length $prefix]} {
			set lp ~
		}
		if {!$ltr} {
			#. below is current prefix with leading tag
			set lp [string reverse $lp]
		}


		catch {array unset vis_hash}
		set cpl [my sw $prefix]


		if {![llength $cpl]} {
			$ltx configure -text "no data"





			incr tf -1
			return
		}

		set startscreen [string equal $prefix {}]
		set lbh [expr {$startscreen ? 1 : 2}]
		set use_shortcut [expr {$startscreen && ([llength $cpl]>8)}]

		if {1} {
			for {set i 0; set tmp {}} {$i<[llength $cpl]} {incr i} {
				foreach {fin x} [lindex $cpl $i] break
				lappend tmp [list [string range $x 0 0] $i]
			}
			#.puts "tfon:tmp=<$tmp>"
			if {[array size uca]} {
				if {[catch {set sorted [lsort -command {my collcomp} -index 0 $tmp]} err]} {

					set sorted [lsort -dictionary -index 0 $tmp]
				}
			} else {
				set sorted [lsort -dictionary -index 0 $tmp]
			}
			#.puts "tfon:sor=<$sorted>"
			set tmp {}
			foreach c0 $sorted {
				foreach {fin x} $c0 break
				lappend tmp [lindex $cpl $x]
			}
			set cpl $tmp
		}
puts T5
		foreach c0 $cpl {
			foreach {final x} $c0 break
			#. first characters are unique keys
			array set vis_hash [list [string range $x 0 0] $c0]
		}
puts T6
		#.destroy {*}[winfo children $vkb]
		my clear $vkb
puts T9
		#. hides breadcrumbs,input



		set colnum 0
		set xx [my column_wise $cpl]

		set row_maxindex 0
		set tmp 1
	#.puts xx=$xx
		foreach y $xx {

			set ylen [llength $y]
			if {$ylen>=$row_maxindex} {
				set row_maxindex $ylen
				set col_maxindex $tmp
			}
			incr tmp
		}
	#.puts "(c,r)=($col_maxindex,$row_maxindex)"
		set rowmax [llength $cpl]
		set xfont $dfont
		set descent [lindex [font metrics $xfont] 3]
		#.set descent [expr {int($descent/2.)}]
		set descent [expr {int($descent)}]
		#. ka_m_el (italic)
		#.incr descent	
		set maxlen 0
		set bc_valid 0
		set lfin [list]

		#. trailing blanks pattern
		set pat_tb {( +)$}
		foreach col $xx {

			#. bg is vertical line between cells
			set flexcol [frame $vkb.c$colnum -bg silver -bd 1]

			incr colnum
			set rownum 0
			set test 1
			foreach c0 $col {
				set c0 [lindex $cpl $c0]
				set fin 0
				set pele ""
				foreach {fin pele} $c0 break
				set dd [string range $pele 0 0]
				set su [string range $pele 1 end]




				set xv $flexcol.r$rownum



				set row [frame $xv -takefocus 1 -bg {}]

				bind $row <Key-Return> [list [self] frameclick $fin $pele]
				#.set row [frame $xv -takefocus 1 -bg \#ededfb]

				incr rownum
				#. below is ahead-completion
				#.set su [string range $pele 1 end]
				set has_pic 0
				if {$fin} {
					set fulltext $prefix$pele
					if {![catch {set pic $pnr($fulltext)} err]} {
						set has_pic 1

					}
				}
				#.puts has_pic=$has_pic
				set ltail ""
				set lpt $lp
				#. sps used as -height value
				#. increase height when wrapped
				set sps $lbh
				if {!$startscreen} {
					if {!$fin} {
						append su \u2026
					} else {
						#. em space, enlarge area to press
						if {[string length $su]<3} {
							append su \u2003
						}
					}
				}
		set trailing_blank 0
		set leading_blank 0
				if {$portrait && !$use_shortcut} {
					set l_spaces ""
					set leading_blank [regexp {^ +} $su l_spaces]
					if {$leading_blank} {
						set rest [string trimleft $su]
					} else {
						set rest $su
					}
					#. puts"leading_blank=$leading_blank, l_spaces=<$l_spaces>"
					#. bug 200117: when split on trailing blank, FingerDown action fails
					set trailing_blank [regexp $pat_tb $rest t_blanks]
					#. puts"### trailing_blank=$trailing_blank, n=<$pele>"
					if {$trailing_blank} {
						set end [expr {[string length $rest]-[string length $t_blanks]}]
						set rest [string range $rest 0 $end]
						#. puts"w/o tb rest=<$rest>"
					}
					set words [split $rest { }]
					set sps [llength $words]
					if {$sps>1} {

						set su $l_spaces
						append su [regsub -all { } $rest \n ]
						#. append su [regsub -all {([^ ]) } $rest \\1\n ]
						#. set su [regsub -all {([^ ]) } $rest \\1\n ]
					#.incr sps -1
						#. if {[string length $l_spaces]} {
						#. 	incr sps -1
						#. }
						#. if {[regexp {^ } $su]} {
						#. 	incr sps -1
						#. }
						set ltail [string repeat \n [expr {$sps-1}]]
						set lpt $lp$ltail
					}
					if {$trailing_blank} {
						append lpt $t_blanks
					}
					if {$sps<$lbh} {
						set sps $lbh
					}
				}

				set actlen [string length $pele]
				if {$startscreen} {
					set sps 1
				} else {
					set lprefix [label $row.lp -font $xfont -text $lpt -fg black -bg $color_back_bg -height $sps]
				}
				#. l0 is the discriminating character
				if {[string length $pele]} {
					scan $dd %c other
					set dd_name [format %x $other]
					append dd $ltail


					#.set l0 [label $row.$dd_name -height $sps -font $xfont -text [subst -nocommands -nobackslashes -novariables $dd] -width 1 -justify center -bg white -fg black]
					set l0 [label $row.$dd_name -height $sps -font $xfont -text $dd -width 1 -justify center -bg $color_d_bg -fg black -takefocus 0]
				} else {
					#.-bg \#ddf9b0 -fg black
					set l0 [label $row.lf -height $sps -font $xfont -text \u23ce -width 1 -justify center \
						-bg $color_d_bg -fg black -takefocus 0]
					set bc_valid 1
					set dd dummy
					set dd_name x
				}

				bind $row <FocusIn> [list $l0 configure -bg hotpink -fg white]

				bind $row <FocusOut> [list $l0 configure -bg $color_d_bg -fg black]



				if {!$ltr} {


					set mp [join [lreverse [split $su \n]] \n]
					set su [string reverse $mp]
					#. reverse wordwise
				}
	#.puts "su=$su"
				if {0} {
				if {!$ltr} {
					set mp [split $su \n]
	#.puts "mp=$mp"
					if {[llength $mp]} {
						set su [join [lreverse $mp] \n]
					}	
	#.puts "su=$su"
				}
				}
				if {[regexp {\n} $su]} {
					set nc 0
					foreach ntemp [split $su \n] {
						set w [string length $ntemp]
						if {$w>$nc} {
							set nc $w
						}
					}
				} else {
					set nc [string length $su]
				}
				incr nc
				set lname $row

				append lname .tail$dd_name

				#. l2 is the leading icon (complete|incomplete)
				if {$fin} {
					if {!$startscreen} {
						#. middot
						set l2 [label $lname -font $xfont -text \ub7$ltail -width 1 -justify center -bg $color_back_bg -fg $uni_fg]
						set kk $prefix
						append kk $pele
						foreach  {{} exval} [array get pnr [my xx_escape $kk]] break
						if {[info exists exval]} {
							lappend lfin [list $exval $lname]
						}
					}
				}  else {
					#. leading tag: vertical ellipsis or middot
					set l2 [label $lname -font $xfont -text \u22ee$ltail -width 1 -justify center -bg $color_back_bg -fg $uni_fg]
				}



				#. pack completion suffix
				#. l0 is discriminating
				if {$startscreen} {
					$l0 configure -width 2
					pack $l0 -side $lr -anchor $an -fill both -expand 1 -ipadx $descent -expand 1
					if {$istouch} {
						bind $l0 <Button-1> +[list [self] frameclick $fin $pele]

					} else {
						bind $l0 <Button-1> +[list [self] frameclick $fin $pele]

					}
				} else {
					if {$prefix_len} {
						pack $l2 -side $lr -anchor $an -fill y
					}
					set l_completion [label $row.con$dd_name -font $xfont -text $su -bg $color_fw_bg -fg black -width $nc -anchor $an -justify $lr -takefocus 0] 
					pack $lprefix -side $lr -anchor $an -fill y
					#. intentionally no expand here
					pack $l0 -side $lr -anchor $an -fill y -ipadx $descent -expand 0
					#. with expand=1 the row background is covered
					#. ev. affects "required size"
					pack $l_completion -side $lr -anchor $an -fill both -expand 1
					if {$istouch} {
						bind $l_completion <Button-1> [list [self] frameclick $fin $pele]
					} else {
						bind $l_completion <Button-1> [list [self] frameclick $fin $pele]

					}
				}
				#. avoid distracting view in language selection
				set skip_pictures [regexp $iso_symbol $prefix]

				if {![string length $kwizact] && !$skip_pictures} {
					if {$has_pic} {


						if {$render_arascr} {
							if {![string is alnum $fulltext]} {
								#. text below pic

								append pic " " [render_arabic $fulltext]
							}
						}
						set pw [string length $pic]
						incr pw
						set pic_l [label $row.pic$dd_name -bg white -fg black -text $pic -font $picfont -width $pw -takefocus 0] 
						pack $pic_l -side $rl -fill both
						#.set fulltext [regsub {\s*} $prefix$pele {}]
						#. putsfulltext=$fulltext
						array set mod [list $pic [my abbrev $fulltext]]
						if {[regexp {^\u2699_..$} $fulltext]} {
							set iso [string range $fulltext end-1 end]
							#. putsiso=$iso
							my tele $iso
							my home
						} else {
							bind $pic_l <Button-1> [list [self] p_speak $pic]
							lappend pics $pic_l
						}
					}
				}

				#. pady shows bg of flexcol
				pack $row -side top -fill both -expand 1 -ipady 2 -ipadx 2
	#.puts "($colnum,$rownum)"
				if {$startscreen} {
					bind $l0 <Button-1> +[list [self] frameclick $fin $pele]
					bind $l0 <Key-Return> [list [self] frameclick $fin $pele]
				} else {
					bind $l2 <Button-1> [list [self] backstep]
					if {$prefix_len} {
						bind $lprefix <Button-1> [list [self] backstep]
						bind $l0 <Button-1> +[list [self] backstep]
						bind $l0 <Key-Return> [list [self] frameclick $fin $pele]
						if {!$startscreen} {
							set loc [list $l_completion $row]
						} else {
							set loc [list $row]
						}
					} else {
						set loc [list $lprefix $l0 $l_completion $row]
					}
					foreach area $loc {
						bind $area <Button-1> [list [self] frameclick $fin $pele]
					}
				}
			}

			#. padx>0 will show background of vkb
			pack $flexcol -side $lr -anchor $an -fill both -expand 1 -padx 0 -ipadx 1
		}


		$ltx configure -height [expr {$portrait ? 2 : 1}]
		sdltk touchtranslate 0
		#. label identical finals
		#. get multiple items
		set mupho [list]
		set prev ""
		foreach lact [lsort -ascii $lfin] {
			foreach {pnum lname} $lact {break}
			if {[string equal $prev $pnum]} {
				lappend mupho $pnum
			}
			set prev $pnum
		}
		#. find cells that lead to equal items
		set lfn 0
		foreach p [lrange [lsort -unique $mupho] 0 9] {
			set lnames [lsearch -exact -index 0 -all $lfin $p]
			foreach muidx $lnames {
				set mulabel [lindex $lfin $muidx]
				set mulabel [lindex $mulabel 1]
				#. mark cells with equal items
				$mulabel configure -text [subst \\u208[format %x $lfn]] -fg black
			}
			incr lfn
		}
puts H4
		#. p is column


		my nada_mem $prefix $dfont
puts K5

		incr tf -1

	}
	method append_suffix {final completion} {

		set current_suffix [string range $completion 1 end]


		set fullkey [my current_prefix]

		append fullkey $completion

		if {$final} {
			pack $aftersuffix -side $lr -after $suffix
			set cc [string range $completion 0 0]
			set pic [my var pnr($fullkey)]
			if {[string equal $pic $kwizact]} {
				set col darkgreen
			} else {
				if {[string length $kwizact]} {
					set col brown
				} else {
					set col black
				}
			}
			$aftersuffix configure -text $pic -fg $col
		} else {
			$aftersuffix configure -text ?
			pack forget $aftersuffix
		}

		if {!$ltr} {
			if {$portrait} {
				set mp [split $current_suffix { }]
				set mplen [llength $mp]
				if {$mplen} {
					set first 1
					set kk [list]
					foreach el $mp {
						if {$first} {
							lappend kk $el
							set first 0
						} else {
							#. indent by 2 spaces
							lappend kk \ \ $el
						}
					}
					set tmp [lreverse $kk]
					set current_suffix [join $tmp \n]
				}	
			}
			#. see adapt_headerfont
			set current_suffix [string reverse $current_suffix]
		} else {
			#. indents
			set mp [split $current_suffix { }]
			set mplen [llength $mp]
			if {$portrait} {
				if {$mplen} {
					set first 1
					set kk [list]
					foreach el $mp {
						if {$first} {
							lappend kk $el
							set first 0
						} else {
							#. indent by 2 spaces
							lappend kk \ \ $el
						}
					}
					set current_suffix [join $kk \n]
				}	
			}	
		}



		$suffix configure -text $current_suffix -bg white -wraplength 0 -justify $lr
		pack $suffix -side $lr -fill x -expand 1 -anchor $an
		pack $aftersuffix -side $lr -fill x -expand 1


		$input icursor end




		set value ""
		if {[catch {append value $pnr($fullkey)} err]} {
			puts "pnr:err=$err"
		}


		#. SEGV my adapt_headerfont
		#. SEGV after idle [list [self] adapt_headerfont]


		set pattern [subst -nocommands ^$iso_symbol\(\[a-z\]\{2,3\}$|..-.*\$\)]
		set dummy ""
		set iso ""
		#.scan $fullkey %c hh
		#.puts [format %04x $hh]


		if {[regexp $pattern $fullkey dummy iso]} {


			my tele $iso
			my home

			return
		}

		if {$final} {

			if {[string length $kwizact]} {

				my kwiz_check $value
			} else {
				if {$kwiztotal} {
					$ltx configure -text [format %d/%d [expr {$kwiztotal-$kwizskipped}] $kwiztotal]
					my kwiz_init
				} else {
					my kwiz_collect $value
				}
			}
		} else {
			#. 030325, else auto_backstep looses prefix
			if {[string length $kwizact]} {

				my kwiz_check $value
			}
		}

	}
	method mylang {{lang {de}}} {

		set mlang $lang
		catch {array unset pnr}
		catch {array unset mod}
		array set pnr {}
		set tmp {}
	set uni {
	\U1f412
	\U1f41c
	\U1f34d
	\U1f34e
	\U1f440
	\U1f441
	\U1f697
	\U1f34c
	\U1f333
	\u26f0
	\U1f37a
	\U1f350
	\U1f968
	\U1f36c
	\U1f453
	\U1f35e
	\U1f4d7
	\u26eb
	\U1f354
	\U1f417
	\U1f418
	\U1f986
	\U1f353
	\U1f95c
	\U1f989
	\U1f987
	\U1f438
	\U1f374
	\U1f992
	\U1f374
	\U1f4d3
	\U1f98c
	\U1f456
	\U1f415
	\U1f994
	\U1f408
	\U1f42b
	\U1f9c0
	\U1f36a
	\U1f352
	\U1f457
	\U1f428
	\U1f40a
	\U1f967
	\U1f424
	\u26df
	\U1f406
	\U1f981
	\U1f401
	\U1f95b
	\U1f43c
	\U1f40e
	\U1f351
	\U1f344
	\U1f427
	\U1f995
	\u26f5
	\u2702
	\U1f40d
	\U1f40c
	\U1f37c
	\U1f45f
	\U1f416
	\U1f953
	\U1f32e
	\u26fe
	\U1f57e
	\U1f405
	\U1f345
	\U1f370
	\U1f69c
	\U1f347
	\U1f996
	\U1f993
	\U1f410
	\U1f6b2
	\U1f40b
	\u2708
	\u270e
	\u2601
	\u263c
	\u270b
	\u2661
	\U1f442
	\U1f41d
	\U1f944
	\U1f445
	\U1f43b
	\U1f95a
	\U1f369
	\U1f42c
	\U1f95e
	\U1f6a2
	\U1f419
	\U1f3dd
	}

	set ru {
	обезьяна
	муравей
	ананас
	яблоко
	глаза
	глаз
	автомобиль
	банан
	дерево
	гора
	пиво
	груша
	кренделек
	конфеты
	очки
	хлеб
	книга
	замок
	бутерброд
	боров
	слон
	утка
	клубничный
	арахис
	филин
	летучая\ мышь
	лягушка
	вилка
	жираф
	нож
	буклет
	олень
	брюки
	собака
	еж
	кот
	верблюд
	сыр
	печенье
	вишня
	платье
	коала
	крокодил
	торт
	цыпленок
	грузовой\ автомобиль
	леопард
	Лев
	мышь
	молоко
	панда
	лошадь
	персиковый
	грибной
	пингвин
	зауроподов
	парусное\ судно
	ножницы
	змея
	улитка
	бутылочки
	башмак
	свинья
	бекон
	тако
	чашка
	телефон
	тигр
	помидор
	пирог
	трактор
	виноград
	т-рекс
	зебра
	коза
	велосипед
	кит
	самолет
	шпилька
	облако
	солнце
	рука
	сердце
	ухо
	пчела
	ложка
	язык
	медведь
	яйцо
	пончик
	дельфин
	блин
	корабль
	Октопус
	остров
	}
	set ru_lower {а б в г д е ё ж з и й к л м н о п р с т у ф х ц ч ш щ ъ ы ь э ю я}




	#



































































































	set ar {
	قرد
	نملة
	أناناس
	تفاحة
	عيون
	عين
	سيارة
	موز
	شجرة
	جبل
	بيرة
	كمثرى
	كعك\ مملح\ و\ جاف
	حلو
	نظارات
	رغيف
	كتاب
	قلعة
	برغر
	خنزير\ ذكر
	فيل
	بطة
	الفراولة
	الفول\ السوداني
	بومة
	مضرب
	ضفدع
	فرع
	زرافة
	سكين
	دفتر
	الغزال
	بنطلون
	الكلب
	قنفذ
	قط
	جمل
	جبنه
	بسكويت
	كرز
	فستان
	الكوال\ دب\ أسترالي
	تمساح
	كيك
	فتاة\ جميلة
	شاحنة\ نقل
	فهد
	أسد
	الفأر
	حليب
	الباندا
	حصان
	خوخ
	فطر
	البطريق
	أقزام
	مركب\ شراعي
	مقص
	ثعبان
	سبيكة
	زجاجة\ الطفل
	حذاء
	خنزير
	لحم\ خنزير\ مقدد
	تاكو
	كوب
	هاتف
	نمر
	طماطم
	كيك
	جرار\ زراعى
	عنب
	تي\ ريكس
	الحمار\ الوحشي
	ماعز
	دراجة
	حوت
	طائرة
	قلم\ جاف
	غيم
	شمس
	كف
	قلب
	إذن
	نحلة
	ملعقة
	لسان
	يتحمل
	بيضة
	الدونات
	دولفين
	الفطائر
	سفينة
	\#oktopus
	جزيرة
	}


	set mn {
	сармагчин
	шоргоолж
	хан\ боргоцой
	\#Apple-ийн
	нүд\ нь
	нүд
	автомашин
	гадил\ жимсний
	мод
	уулын
	шар\ айраг
	лийрийн
	\#pretzel
	чихэр
	нүдний\ шил
	талх
	ном
	цайз
	бургер
	гахай
	заан
	нугас
	гүзээлзгэний
	газрын\ самрын
	шар\ шувуу
	Бат
	мэлхий
	сэрээ
	анааш
	хутга
	товхимол
	буга
	өмд
	нохой
	зараа
	муур
	тэмээний
	бяслаг
	жигнэмэг
	интоор
	хувцас
	\#Koala
	матар
	бялуу
	дэгдээхэйн
	тэрэгний
	ирвэс
	Лео
	хулгана
	сүү
	панда
	морь
	тоорын
	мөөг
	оцон\ шувуу
	зауроподын
	далбаат
	хайч
	могой
	эмгэн\ хумсны
	\#schoppen
	гутлын
	гахайн
	гахайн
	\#Taco
	аяга
	гар\ утас
	\#Tiger
	улаан\ лоолийн
	бялуу
	трактор
	усан\ үзэм
	\#T-Rex
	тахө
	ямааны
	дугуй
	халим
	нисэх\ онгоц
	зүү
	үүл
	нар
	гар
	зүрх
	чих
	зөгий
	халбага
	хэл
	баавгай
	өндөг
	гурилан\ бүтээгдэхүүний
	далайн\ гахай
	Хуушуурын
	хөлөг\ онгоц
	\#oktopus
	арал
	}

	set uz {
	}


	set pa {
	}

	set de_lower {a b c d e f g h i j k l m n o p q r s t u v w x y z ä ö ü}
	set de {
	affe
	ameise
	ananas
	apfel
	augen
	auge
	auto
	banane
	baum
	berg
	bier
	birne
	brezel
	bonbon
	brille
	brot
	buch
	burg
	burger
	wildschwein
	elefant
	ente
	erdbeere
	erdnuss
	eule
	fledermaus
	frosch
	gabel
	giraffe
	messer
	heft
	hirsch
	hose
	hund
	igel
	katze
	kamel
	käse
	keks
	kirschen
	kleid
	koala
	krokodil
	kuchen
	küken
	lastwagen
	leopard
	löwe
	maus
	milch
	panda
	pferd
	pfirsich
	pilz
	pinguin
	sauropod
	segelboot
	scheere
	schlange
	schnecke
	schoppen
	schuh
	schwein
	speck
	taco
	tasse
	telefon
	tiger
	tomate
	torte
	traktor
	trauben
	t-rex
	zebra
	ziege
	fahrrad
	wal
	flugzeug
	stift
	wolke
	sonne
	hand
	herz
	ohr
	biene
	löffel
	zunge
	bär
	ei
	donut
	delfin
	pfannkuchen
	schiff
	oktopus
	insel
}
set kin {
inkende
ibimonyo
inanasi
pome
amaso
ijisho
imodoka
igitoki
igiti
umusozi
byeri
amapera
pretzel
bonbon
ibirahure
umutsima
igitabo
igihome
burger
ingurube
inzovu
inkongoro
strawberry
ibishyimbo
igihunyira
bat
igikeri
fork
giraffe
icyuma
agatabo
impongo
ipantaro
imbwa
uruziga
injangwe
ingamiya
foromaje
ibisuguti
cherry
imyambarire
koala
ingona
cake
inkoko
ikamyo
ingwe
intare
imbeba
amata
panda
ifarashi
amashaza
ibihumyo
penguin
sauropod
ubwato
imikasi
inzoka
igituba
icupa
inkweto
ingurube
bacon
taco
igikombe
telefone
ingwe
inyanya
pie
romoruki
inzabibu
t-rex
zebra
ihene
igare
baleine
indege
pin
igicu
izuba
ukuboko
umutima
ugutwi
inzuki
ikiyiko
ururimi
idubu
amagi
impano
dolphin
pancakes
ubwato
octopus
ikirwa
}
set uz {
maymun
chumoli
ananas
olma
ko'zlar
ko'z
avtomatik
banan
daraxt
tog\ '
pivo
nok
pretzel
tonna
ko'zoynaklar
non
kitob
qasr
burger
yovvoyi\ cho'chqa
fil
o'rdak
qulupnay
yerfıstığı
boyqush
yarasa
qurbaqa
vilkalar
jirafa
pichoq
risola
kiyik
shimlar
it
kirpi
mushuk
tuya
pishloq
pechene
gilos
kiyinish
koala
timsoh
tort
tovuq
yuk\ mashinasi
leopard
sher
sichqoncha
sut
panda
ot
shaftoli
qo'ziqorin
pingvin
sauropod
yelkanli\ qayiq
qaychi
ilon
salyangoz
chaqaloq\ shishasi
poyafzal
cho'chqa
pastırma
tako
kubok
telefon
yo'lbars
pomidor
pirog
traktorchi
uzum
t-reks
zebra
echki
velosiped
kit
tekislik
pin
bulut
quyosh
qo'l
yurak
quloq
asalari
qoshiq
til
ayiq
tuxum
pichoq
delfin
krep
kema
ahtapot
orol
}
set or {
ମାଙ୍କଡ଼
ପିମ୍ପୁଡ଼ି
ଧନିଆ\ ପତ୍ର\ |
ଆପଲ୍
ଆଖି
ଆଖି
ଅଟୋ
କଦଳୀ
ଗଛ
ପର୍ବତ
ବିୟର
ମୋତି
\#pretzel
ବୋନବନ୍\ |
ଚଷମା
ରୁଟି\ |
ପୁସ୍ତକ
ଦୁର୍ଗ
ବରଗଡ
ବଣୁଆ\ ଘୁଷୁରୀ\ |
ହାତୀ
ଡାକୁ
ଷ୍ଟ୍ରବେରୀ\ |
କଦଳୀ
ପେଚା
ବ୍ୟାଟ୍
ବେଙ୍ଗ
କଣ୍ଟା
ଜିରାଫ୍\ |
ଛୁରୀ
ବୁକଲେଟ୍
ହରିଣ
ପ୍ୟାଣ୍ଟ
କୁକୁର
ହେଜ୍
ବିଲେଇ
ଓଟ
ପନିର
ବିସ୍କୁଟ\ |
ଚେରି
ପୋଷାକ
କୋଲା
କୁମ୍ଭୀର
କେକ୍
କୁକୁଡ଼ା
ଟ୍ରକ୍
ଚିତାବାଘ
ସିଂହ
ମାଉସ୍
କ୍ଷୀର
ପଣ୍ଡା
ଘୋଡା
ପିଚ
ମୂଷା
ପେଙ୍ଗୁଇନ୍\ |
\#sauropod
ନାବିକ
କଞ୍ଚା
ସାପ
ଶାବକ
ଶିଶୁ\ ବୋତଲ\ |
ଜୋତା
ଘୁଷୁରି
ବେକନ୍\ |
ଟାକୋ\ |
କପ୍
ଫୋନ୍\ |
ବାଘ
ଟମାଟୋ\ |
ପିଠା
ଟ୍ରାକ୍ଟର
ଅଙ୍ଗୁର
\#t-rex
ଜେବ୍ରା
ଛେଳି
ସାଇକେଲ
ବାଘ
ବିମାନ
ପିନ୍
ମେଘ
ସୂର୍ଯ୍ୟ
ହାତ
ହୃଦୟ
କାନ
ମହୁମାଛି
ଚାମଚ
ଜିଭ
ଭାଲୁ
ଅଣ୍ଡା
ଦାନ
ଡଲଫିନ୍
ପାନକେକ୍\ |
ଜାହାଜ
ଅଷ୍ଟପାସ୍
ଦ୍ୱୀପ
}
set pt {
macaco
formiga
abacaxi
maça
olhos
olho
auto
banana
arvore
montanha
cerveja
pera
pretzel
bombom
óculos
pão
livro
castelo
hamburguer
javali
elefante
pato
morango
amendoim
coruja
morcego
sapo
garfo
girafa
faca
livreto
veado
calças
cão
ouriço
gato
camelo
queijo
biscoito
cerejas
vestido
coala
crocodilo
bolo
pintinho
caminhão
leopardo
leão
rato
leite
panda
cavalo
pêssego
cogumelo
pinguim
saurópode
veleiro
tesoura
cobra
caracol
mamadeira
sapato
porco
bacon
taco
copo
telefone
tigre
tomate
torta
trator
uvas
t-rex
zebra
cabra
bicicleta
baleia
avião
alfinete
nuvem
sol
mão
coração
ouvido
abelha
colher
língua
suportar
ovo
rosquinha
golfinho
panquecas
navio
polvo
ilha
}
	set it {
scimmia
formica
ananas
mela
occhi
occhio
auto
banana
albero
montagna
birra
pera
salatino
caramella
occhiali
pane
libro
castello
hamburger
cinghiale
elefante
anatra
fragola
arachide
gufo
pipistrello
rana
forchetta
giraffa
coltello
libretto
cervo
pantaloni
cane
riccio
gatto
cammello
formaggio
biscotto
ciliegie
abito
koala
coccodrillo
torta
pulcino
camion
leopardo
Leone
mouse
latte
panda
cavallo
pesca
fungo
pinguino
sauropode
barca\ a\ vela
forbici
serpente
lumaca
biberon
scarpa
maiale
bacon
taco
tazza
telefono
tigre
pomodoro
torta
trattore
uva
t-rex
zebra
capra
bicicletta
balena
aereo
perno
nuvola
sole
mano
cuore
orecchio
ape
cucchiaio
linguetta
orso
uovo
ciambella
delfino
pancake
nave
oktopus
isola
}
set es {
	mono
hormiga
piña
manzana
ojos
ojo
auto
plátano
arbol
montaña
cerveza
pera
pretzel
bombón
gafas
pan
libro
castillo
hamburguesa
jabalí
elefante
pato
fresa
maní
búho
murciélago
rana
tenedor
jirafa
cuchillo
folleto
ciervo
pantalones
perro
erizo
gato
camello
queso
galleta
cerezas
vestido
koala
cocodrilo
pastel
chica
camion
leopardo
leon
ratón
la\ leche
panda
caballo
durazno
seta
pinguino
saurópodo
velero
tijeras
serpiente
caracol
biberón
zapato
cerdo
tocino
taco
taza
telefono
tigre
tomate
pastel
tractor
uvas
t-rex
cebra
cabra
bicicleta
ballena
avion
pin
nube
sol
mano
corazon
oreja
abeja
cuchara
lengua
soportar
huevo
rosquilla
delfín
panqueques
barco
pulpo
isla
}
	set fr {
	singe
	fourmi
	ananas
	pomme
	yeux
	œil
	voiture
	banane
	arbre
	montagne
	bière
	poire
	bretzel
	bonbon
	des\ lunettes
	pain
	livre
	château
	burger
	sanglier
	l'éléphant
	canard
	fraise
	cacahuète
	hibou
	chauve\ souris
	grenouille
	fourchette
	girafe
	couteau
	carnet
	cerf
	pantalon
	chien
	hérisson
	chat
	chameau
	fromage
	biscuit
	cerises
	robe
	koala
	crocodile
	gâteau
	poussin
	camion
	léopard
	lion
	souris
	lait
	panda
	cheval
	pêche
	champignon
	manchot
	sauropode
	voilier
	les\ ciseaux
	serpent
	escargot
	biberon
	chaussure
	porc
	bacon
	taco
	tasse
	téléphone
	tigre
	tomate
	gâteau
	tracteur
	les\ raisins
	t-rex
	zèbre
	chèvre
	vélo
	baleine
	avion
	stylo
	nuage
	soleil
	main
	cœur
	oreille
	abeille
	cuillère
	langue
	ours
	oeuf
	donut
	dauphin
	crêpes
	navire
	oktopus
	île
	}
	set en_lower {a b c d e f g h i j k l m n o p q r s t u v w x y z}
	set en {
	monkey
	ant
	pineapple
	apple
	eyes
	eye
	automobile
	banana
	tree
	mountain
	beer
	pear
	pretzel
	sweet
	glasses
	loaf
	book
	castle
	burger
	boar
	elephant
	duck
	strawberry
	peanut
	owl
	bat
	frog
	fork
	giraffe
	knife
	notebook
	deer
	trousers
	dog
	hedgehog
	cat
	camel
	cheese
	biscuit
	cherries
	dress
	koala
	crocodile
	cake
	chick
	truck
	leopard
	lion
	mouse
	milk
	panda
	horse
	peach
	mushroom
	penguin
	sauropod
	sailboat
	scissors
	snake
	slug
	baby\ bottle
	shoe
	pig
	bacon
	taco
	cup
	phone
	tiger
	tomato
	cake
	tractor
	grapes
	t-rex
	zebra
	goat
	bicycle
	whale
	plane
	pen
	cloud
	sun
	hand
	heart
	ear
	bee
	spoon
	tongue
	bear
	egg
	donut
	dolphin
	pancakes
	ship
	oktopus
	island
	}
	#. yoruba
	#. yor fails too
	set yo {
	obo
	kokoro
	ope\ oyinbo
	apu
	oju
	oju
	ọkọ\ ayọkẹlẹ
	ogede
	igi
	òkè
	oti\ bia
	eso\ pia
	díb.
	adun
	gilaasi
	buredi
	iwe
	ile\ odi
	olè
	ariwo
	erin
	pepeye
	iru\ eso\ didun\ kan
	epa
	owiwi
	adan
	Ọpọlọ
	orita
	agunbo
	ọbẹ
	ajako
	agbọnrin
	awọn\ sokoto
	aja
	hedgehog
	o\ nran
	ràkúnmí
	warankasi
	akara\ oyinbo
	awọn\ ṣẹẹri
	aṣọ
	kola
	ooni
	akara\ oyinbo
	adiye
	oko\ nla
	adẹtẹ
	kiniun
	asin
	wara
	panda
	ẹṣin
	eso\ pishi
	osun
	penguin
	sauropod
	ọkọ\ oju-omi\ kekere
	scissors
	ejo
	sísọ
	ìgo\ ọmọ
	bata
	ẹlẹdẹ
	bekin\ eran\ elede
	taco
	ife
	foonu
	ẹyẹ
	tomati
	akara\ oyinbo
	tirakito
	àjàrà
	t-rex
	kẹtẹkẹtẹ\ abila
	ewurẹ
	keke
	ẹja
	ọkọ\ ofurufu
	ikọwe
	awọsanma
	oorun
	ọwọ
	obi
	eti
	oyin
	sibi
	ahọn
	jẹri
	ẹyin
	donut
	ẹja\ abirun
	awọn\ oyinbo
	ọkọ\ oju-omi
	oktopus
	erekusu
	}
	#. sw did not work
	set swa {
	tumbili
	chungu
	mananasi
	apple
	macho
	jicho
	gari
	ndizi
	mti
	mlima
	bia
	peari
	pretzel
	tamu
	glasi
	mkate
	kitabu
	ngome
	burger
	boar
	tembo
	bata
	sitirishi
	karanga
	bundi
	bat
	chura
	uma
	twiga
	kisu
	daftari
	kulungu
	suruali
	mbwa
	hedgehog
	paka
	ngamia
	jibini
	biskuti
	cherries
	nguo
	koala
	mamba
	keki
	kifaranga
	lori
	chui
	simba
	panya
	maziwa
	panda
	farasi
	peach
	uyoga
	penguin
	sauropod
	mashua
	mkasi
	nyoka
	konokono
	chupa\ ya\ mtoto
	kiatu
	nguruwe
	Bacon
	taco
	kikombe
	simu
	tiger
	nyanya
	keki
	trekta
	zabibu
	t-rex
	zebra
	mbuzi
	baiskeli
	nyangumi
	ndege
	kalamu
	wingu
	jua
	mkono
	moyo
	sikio
	nyuki
	kijiko
	ulimi
	kubeba
	yai
	donut
	pomboo
	pancakes
	meli
	oktopus
	kisiwa
	}
	set zu {
	inkawu
	intuthwane
	uphayinaphu
	i-apula
	amehlo
	iso
	imoto
	ibhanana
	isihlahla
	intaba
	ubhiya
	ipheya
	i-pretzel
	limnandi
	izibuko
	isinkwa
	incwadi
	inqaba
	burger
	izingulube
	indlovu
	idada
	ijikijolo
	ithanga
	isikhova
	bat
	isele
	imfoloko
	indlulamithi
	ummese
	incwadi\ yokubhalela
	inyamazane
	ibhulukwe
	inja
	i-hedgehog
	ikati
	ikamela
	ushizi
	ibhisikidi
	cherries
	gqoka
	koala
	ingwenya
	ikhekhe
	inkukhu
	iloli
	ingwe
	ibhubesi
	igundane
	ubisi
	panda
	ihhashi
	ipentshisi
	ikhowe
	penguin
	i-sauropod
	isikebhe\ sesikebhe
	isikhafu
	inyoka
	i-slug
	ibhodlela\ lengane
	isicathulo
	ingulube
	i-bacon
	i-taco
	inkomishi
	ifoni
	ihlosi
	utamatisi
	ikhekhe
	ugandaganda
	amagilebhisi
	t-rex
	zebra
	imbuzi
	ibhayisekili
	umkhomo
	indiza
	ipeni
	ifu
	ilanga
	isandla
	inhliziyo
	izindlebe
	inyosi
	ukhezo
	ulimi
	ibhere
	iqanda
	donut
	dolphin
	ama-pancake
	umkhumbi
	oktopus
	isiqhingi
	} 
	set pl {
	małpa
	mrówka
	ananas
	jabłko
	oczy
	oko
	samochód
	banan
	drzewo
	góra
	piwo
	gruszka
	precel
	słodkie
	okulary
	bochenek
	książka
	zamek
	burger
	dzik
	słoń
	kaczka
	truskawka
	arachid
	sowa
	nietoperz
	żaba
	widelec
	żyrafa
	nóż
	notatnik
	jeleń
	spodnie
	pies
	jeż
	kot
	wielbłąd
	ser
	herbatnik
	wiśnie
	sukienka
	koala
	krokodyl
	ciasto
	pisklę
	ciężarówka
	lampart
	lew
	mysz
	mleko
	panda
	koń
	brzoskwinia
	grzyb
	pingwin
	zauropod
	żaglówka
	nożyce
	wąż
	ślimak
	butelka\ dla\ dziecka
	but
	świnia
	boczek
	taco
	puchar
	telefon
	tygrys
	pomidor
	ciasto
	ciągnik
	winogrona
	t-rex
	zebra
	koza
	rower
	wieloryb
	samolot
	długopis
	chmura
	słońce
	dłoń
	serce
	ucho
	pszczoła
	łyżka
	język
	niedźwiedź
	jajko
	pączek
	delfin
	naleśniki
	statek
	oktopus
	wyspa
	}
	set tam {
	குரங்கு
	எறும்பு
	அன்னாசி
	ஆப்பிள்
	கண்கள்
	கண்
	ஆட்டோமொபைல்
	வாழை
	மரம்
	மலை
	பீர்
	பேரிக்காய்
	கண்களை\ அகல
	இனிப்பு
	கண்ணாடிகள்
	ரொட்டி
	நூல்
	கோட்டைக்கு
	பர்கர்
	பன்றி
	யானை
	வாத்து
	ஸ்ட்ராபெரி
	வேர்க்கடலை
	ஆந்தை
	பேட்
	தவளை
	முள்\ கரண்டி
	ஒட்டகச்சிவிங்கி
	கத்தி
	நோட்புக்
	மான்
	கால்சட்டை
	நாய்
	ஹெட்ஜ்ஹாக்
	பூனை
	ஒட்டக
	பாலாடைக்கட்டி
	பிஸ்கட்
	செர்ரிகளில்
	உடை
	கோலா
	முதலை
	கேக்
	குஞ்சு
	டிரக்
	சிறுத்தை
	சிங்கம்
	சுட்டி
	பால்
	பாண்டா
	குதிரை
	பீச்
	காளான்
	பென்குவின்
	\#sauropod
	பாய்மரப்
	கத்தரிக்கோல்
	பாம்பு
	அடி
	குழந்தை\ \ பாட்டில்
	ஷூ
	பன்றி
	பன்றி\ இறைச்சி
	டகோ
	கோப்பை
	தொலைபேசி
	புலி
	தக்காளி
	கேக்
	டிராக்டர்
	திராட்சை
	டி-ரெக்ஸ்
	வரிக்குதிரை
	வெள்ளாடு
	மிதிவண்டி
	திமிங்கிலம்
	விமானம்
	பேனா
	மேகம்
	சூரியன்
	கை
	இதயம்
	காது
	தேனீ
	கரண்டியால்
	நாக்கு
	தாங்க
	முட்டை
	கோளாறு
	டால்பின்
	அப்பத்தை
	கப்பல்
	\#oktopus
	தீவின்
	}
	#. igbo  ig ibo
	#. verify on igbotique.com/dict/ig
	set ibo {
	enwe
	ndanda
	ọkwụrụ\ bekee
	apụl
	anya
	anya
	ugboala
	unere
	osisi
	ugwu
	biya
	ube
	pretzel
	ụtọ
	iko
	ogbe\ achịcha
	akwụkwọ
	obí\ eze
	burger
	boar
	enyi
	ọbọgwụ
	strọberị
	ahuekere
	ikwiikwii
	bat
	frog
	ndụdụ
	giray
	mma
	akwụkwọ\ ndetu
	mgbada
	eriri\ afọ
	nkịta
	mgbochi
	pusi
	kamel
	chiiz
	bisikiiti
	cherị
	akwa
	koala
	agụ\ iyi
	achicha
	nwa\ nnụnụ
	ugboala
	agụ
	ọdụm
	òké
	mmiri\ ara
	panda
	inyinya
	piich
	ero
	penguuin
	sauropod
	ugbo\ mmiri
	mkpa
	agwo
	slug
	karama\ \ nwa
	akpụkpọ\ ụkwụ
	ezi
	anụ\ ezi
	taco
	iko
	ekwentị
	agu
	Tomato
	achicha
	traktọ
	mkpụrụ\ vaịn
	t-rex
	zebra
	mkpi
	igwe
	whale
	ugbo\ elu
	pen
	igwe\ ojii
	anwụ
	aka
	obi
	ntị
	anu
	ngaji
	ire
	ibu
	akwa
	donut
	dolphin
	pancakes
	ụgbọ\ mmiri
	oktopus
	agwaetiti
	}

































































































	set fa {
	میمون
	مورچه
	آناناس
	سیب
	چشم\ ها
	چشم
	خودرو
	موز
	درخت
	کوه
	آبجو
	گلابی
	پیشران
	شیرین
	عینک
	نان
	کتاب
	قلعه
	همبرگر
	گراز
	فیل
	اردک
	توت\ فرنگی
	بادام\ زمینی
	جغد
	خفاش
	قورباغه
	چنگال
	زرافه
	چاقو
	نوت\ بوک
	گوزن
	شلوار
	سگ
	جوجه\ تيغي
	گربه
	شتر
	پنیر
	بیسکویت
	گیلاس
	لباس
	کوالا
	تمساح
	کیک
	جوجه
	کامیون
	پلنگ
	شیر\ نر
	ماوس
	شیر
	پاندا
	اسب
	هلو
	قارچ
	پنگوئن
	ساروپود
	قایق\ بادبانی
	قیچی
	مار
	خزنده
	کودک\ بطری
	کفش
	خوک
	بیکن
	ساندویچ\ مکزیکی
	فنجان
	تلفن
	ببر
	گوجه\ فرنگی
	کیک
	تراکتور
	انگور
	تی\ رکس
	گورخر
	بز
	دوچرخه
	نهنگ
	سطح
	خودکار
	ابر
	آفتاب
	دست
	قلب
	گوش
	زنبور\ عسل
	قاشق
	زبان
	خرس
	تخم\ مرغ
	دونات
	دلفین
	پنکیک
	کشتی
	اختاپوس
	جزیره
	}
	#. pashto
	set ps {
	بندر
	میږی
	انگور
	صوف
	اکيون
	اک
	گاڏين\ جو
	ڪيانا
	وڻ
	جبل
	بيئر
	ڀٽائي\ ٿو
	نون
	مٺي
	شيشي
	ډوډۍ
	ڪتاب
	قلعو
	برگر
	ڇڏيندڙ
	هاٿي
	بتڪ
	ڏاڪڻ
	مٺو
	اٽل
	بٽ
	ڏيڏر
	ڪٽڪ
	جفا
	چاقو
	نوٽ\ بڪ
	منير
	پتلون
	ڪتو
	هيڊڙا
	ٻلي
	اٺ
	پنيس
	بسٽيو
	مرچ
	لباس
	ڪوالا
	زراعت
	ڪيڪ
	ڇوڪري
	ٽرڪ
	ڪوڙا
	شعر
	ڪوئو
	کير
	پانڈا
	گهوڙي
	آڙو
	نوڪري
	پينگوئن
	سوروپپو
	سريل\ بوٽ
	اسڪسيس
	نانگ
	گندو
	ٻار\ جي\ بوتل
	جوتن
	سور
	باڪس
	ٽڪو
	پيالو
	فون
	شگر
	ٽماٽو
	ڪيڪ
	ٽريڪٽر
	انگور
	\#t-\ rex
	زيبرا
	بکري
	سائيڪل
	وهن
	جهاز
	قلم
	ڪڪر
	سج
	هٿ
	دل
	ڪن
	مک
	چمچو
	زبان
	کڻندو
	هيڪو
	ڊونٽ
	ڊولفن
	پينڪڪس
	ٻيڙي
	اوپيٽيوس
	ٻيٽ
	}
	set fi {
	apina
	muurahainen
	ananas
	omena
	katse
	silmä
	auto
	banaani
	puu
	vuori
	olut
	päärynä
	suolarinkilä
	makea
	lasit
	leipä
	kirja
	linna
	hampurilainen
	villisika
	norsu
	ankka
	mansikka
	maapähkinä
	pöllö
	lepakko
	sammakko
	haarukka
	kirahvi
	veitsi
	muistikirja
	peura
	housut
	koira
	siili
	kissa
	kameli
	juusto
	keksi
	kirsikat
	mekko
	koala
	krokotiili
	kakku
	tipu
	kuorma-auto
	leopardi
	leijona
	hiiri
	maito
	panda
	hevonen
	persikka
	sieni
	pingviini
	sauropod
	purjevene
	sakset
	käärme
	etana
	\#schoppen
	kenkä
	sika
	pekoni
	taco
	kuppi
	puhelin
	tiikeri
	tomaatti
	kakku
	traktori
	viinirypäleet
	t-rex
	seepra
	vuohi
	polkupyörä
	valas
	lentokone
	kynä
	pilvi
	aurinko
	käsi
	sydän
	korva
	mehiläinen
	lusikka
	kieli
	karhu
	muna
	donitsi
	delfiini
	pannukakut
	alus
	oktopus
	saari
	}
	set hu {
	majom
	hangya
	ananász
	alma
	szemek
	szem
	autó
	banán
	fa
	hegy
	sör
	körte
	perec
	édes
	szemüveg
	cipó
	könyv
	vár
	burger
	vaddisznó
	elefánt
	kacsa
	eper
	földimogyoró
	bagoly
	denevér
	béka
	villa
	zsiráf
	kés
	jegyzetfüzet
	szarvas
	nadrág
	kutya
	sündisznó
	macska
	teve
	sajt
	keksz
	cseresznye
	ruha
	koala
	krokodil
	torta
	csaj
	kamion
	leopárd
	oroszlán
	egér
	tej
	pandamackó
	ló
	őszibarack
	gomba
	pingvin
	sauropod
	vitorlás
	olló
	kígyó
	meztelen\ csiga
	\#SCHOPPEN
	cipő
	malac
	szalonna
	taco
	csésze
	telefon
	tigris
	paradicsom
	torta
	traktor
	szőlő
	t-Rex
	zebra
	kecske
	kerékpár
	bálna
	repülőgép
	toll
	felhő
	nap
	kéz
	szív
	fül
	méh
	kanál
	nyelv
	medve
	tojás
	fánk
	delfin
	palacsinta
	hajó
	oktopus
	sziget
	}
	set nl {
	aap
	mier
	ananas
	appel
	ogen
	oog
	auto
	banaan
	boom
	berg
	bier
	peer
	krakeling
	snoep
	bril
	brood
	boek
	kasteel
	hamburger
	mannetjesvarken
	olifant
	eend
	aardbei
	pinda
	uil
	knuppel
	kikker
	vork
	giraffe
	mes
	notitieboekje
	hert
	broek
	hond
	egel
	kat
	kameel
	kaas
	biscuit
	kersen
	jurk
	koala
	krokodil
	cake
	kuiken
	vrachtauto
	luipaard
	leeuw
	muis
	melk
	panda
	paard
	perzik
	paddestoel
	pinguïn
	sauropod
	zeilboot
	schaar
	slang
	slak
	schoppen
	schoen
	varken
	spek
	taco
	kop
	telefoon
	tijger
	tomaat
	taart
	trekker
	druiven
	t-rex
	zebra
	geit
	fiets
	walvis
	vliegtuig
	pen
	wolk
	zon
	hand
	hart
	oor
	bij
	lepel
	tong
	beer
	ei
	donut
	dolfijn
	pannekoeken
	schip
	oktopus
	eiland
	}

































































































	set ur {
	بندر
	چیونٹی
	انناس
	سیب
	آنکھیں
	آنکھ
	گاڑی
	کیلا
	درخت
	پہاڑ
	بیئر
	ناشپاتی
	\#pretzel
	میٹھا
	شیشے
	روٹی
	کتاب
	قلعہ
	برگر
	سوار
	ہاتھی
	بطخ
	اسٹرابیری
	مونگفلی
	اللو
	چمگادڑ
	مینڈک
	کانٹا
	جراف
	چاقو
	کاپی
	ہرن
	پتلون
	کتا
	ہیج\ ہاگ
	کیٹ
	اونٹ
	پنیر
	بسکٹ
	چیری
	لباس
	کوآلا
	مگرمچھ
	کیک
	چھوٹا
	ٹرک
	چیتے
	شیر
	ماؤس
	دودھ
	پانڈا
	گھوڑا
	آڑو
	مشروم
	پینگوئن
	\#sauropod
	سیل\ بوٹ
	قینچی
	سانپ
	سلگ
	بچے\ کی\ بوتل
	جوتا
	سور
	بیکن
	ٹیکو
	کپ
	فون
	چیتا
	ٹماٹر
	کیک
	ٹریکٹر
	انگور
	ٹی\ ریکس
	زیبرا
	بکرا
	بائیسکل
	وہیل
	ہوائی\ جہاز
	قلم
	بادل
	سورج
	ہاتھ
	دل
	کان
	مکھی
	چمچ
	زبان
	ریچھ
	انڈہ
	ڈونٹ
	ڈالفن
	پینکیکس
	جہاز
	اوکٹوپس
	جزیرہ
	}
	#. //glosbe.com

	set yi {
	מאַלפּע
	מוראַשקע
	אַנאַנאַס
	עפּל
	אויגן
	אויג
	קאר
	באַנאַנע
	בוים
	באַרג
	ביר
	באַרנע
	פּרעטזעל
	זיס
	ברילן
	לאַבן
	בוך
	שלאָס
	בערגער
	קאַבאַן
	העלפאַנד
	קאַטשקע
	סטראָבערי
	פיסטאַשקע
	סאָווע
	פלעדערמויז
	זשאַבע
	גאָפּל
	דזשעראַף
	מעסער
	העפט
	הירש
	הויזן
	הונט
	שטעכלער
	קאַץ
	קעמל
	קעז
	ביסקאַט
	טשעריז
	קלייד
	קאָאַלאַ
	קראָקאָדיל
	שטיקל
	פייגעלע
	טראָק
	לעמפּערט
	לייב
	מויז
	מילך
	פּאַנדאַ
	פערד
	פערשקע
	שוועמל
	פּינגווין
	סאַוראָפּאָד
	זעגלשיף
	שער
	שלאַנג
	שנעק
	סטשאָפּפּען
	שוך
	כאַזער
	בייקאָן
	טאַקאָ
	גלעזל
	טעלעפאָן
	טיגער
	פּאָמידאָר
	פּיראָג
	טראַקטאָר
	טרויבן
	ה-רעקס
	זעברע
	ציג
	וועלאָסיפּעד
	וואַלפיש
	אַעראָפּלאַן
	פעדער
	וואָלקן
	זון
	האַנט
	האַרץ
	אויער
	בין
	לעפל
	צונג
	בער
	יי
	פּעמפּיקל
	דעלפין
	פּאַנקייקס
	שיף
	ספּרוט
	אינזל
	}
	set iw {
	קוף
	נמלה
	אננס
	תפוח\ עץ
	עיניים
	עין
	רכב
	בננה
	עץ
	הר
	בירה
	אגס
	בייגלה
	מתוק
	משקפיים
	כיכר
	ספר
	טירה
	המבורגר
	חזיר\ בר
	פיל
	ברווז
	תות
	בוטן
	ינשוף
	עטלף
	צפרדע
	מזלג
	ג'ירפה
	סכין
	מחברת
	צבי
	מכנסיים
	כלב
	קיפוד
	חתול
	גמל
	גבינה
	ביסקוויט
	דובדבנים
	שמלה
	קואלה
	תנין
	עוגה
	אפרוח
	משאית
	נמר
	אריה
	עכבר
	חלב
	פנדה
	סוס
	אפרסק
	פטרייה
	פינגווין
	\#sauropod
	מפרשית
	מספריים
	נחש
	שבלול
	שופן
	נעל
	חזיר
	בייקון
	טאקו
	כוס
	מכשיר\ טלפון
	נמר
	עגבנייה
	עוגה
	טרקטור
	ענבים
	טי\ רקס
	זברה
	עז
	אופניים
	לוויתן
	מטוס
	עט
	ענן
	שמש
	יד
	לב
	אוזן
	דבורה
	כף
	לשון
	דוב
	ביצה
	סופגניה
	דולפין
	פנקייקים
	ספינה
	תמנון
	אי
	}

	set mr {
	माकड
	मुंगी
	अननस
	सफरचंद
	डोळे
	डोळा
	कार
	केळी
	झाड
	डोंगरावर
	बिअर
	\#PEAR,
	खारे\ बिस्कीट
	कँडी
	चष्मा
	भाकरी
	पुस्तक
	किल्लेवजा\ वाडा
	बर्गर
	डुक्कर
	हत्ती
	परतले
	छोटी
	शेंगदाणा
	घुबड
	फलंदाजी
	बेडूक
	काटा
	जिराफ
	चाकू
	पुस्तिका
	हरण
	पायघोळ
	कुत्रा
	\#hedgehog
	मांजर
	उंट
	चीज
	बिस्कीट
	\#cherries
	ड्रेस
	कोलाला
	मगर
	केक
	चिक
	ट्रक
	चित्ता
	लिओ
	माऊस
	दूध
	पांडा
	घोडा
	सुदंर\ आकर्षक\ मुलगी
	मशरूम
	पेंग्विन
	\#sauropod
	\#sailboat
	कात्री
	साप
	गोगलगाय
	\#schoppen
	जोडा
	डुक्कर
	खारवून\ वाळवलेले\ डुकराचे\ मांस
	असावा
	कप
	फोन
	वाघ
	टोमॅटो
	पाई
	ट्रॅक्टर
	द्राक्षे
	टी-राजा
	झेब्रा
	शेळी
	दुचाकी
	व्हेल
	विमान
	पिन
	ढग
	सूर्य
	हात
	हृदय
	कान
	मधमाशी
	चमच्याने
	जीभ
	अस्वल
	अंडी
	मिठाई
	डॉल्फिन
	\#pancake
	जहाज
	\#oktopus
	बेट
	}
	#. from wiki/Devanagari
	#. 
	#.set dev_con {
	#.क ख ग घ ङ च छ ज झ ञ ट ठ ड ढ ण त थ द ध न प फ ब भ म य र ल व श ष स ह
	#.}
	#. vowels from wiki/Marathi_Language#Devanagari
	#.set dev_vow {
	#.अ आ इ ई उ ऊ ए ऐ ओ औ अं अः ॲ ऑ 
	#.}
	set mr_lower {क ख ग घ ङ च छ ज झ ञ ट ठ ड ढ ण त थ द ध न प फ ब भ म य र ल व श ष स ह अ आ इ ई उ ऊ ए ऐ ओ औ अं अः ॲ ऑ }

	set hi {
	बंदर
	चींटी
	अनानास
	सेब
	आंखें
	आंख
	कार
	केला
	पेड़
	पहाड़
	बियर
	नाशपाती
	एक\ प्रकार\ की\ रोटी
	कैंडी
	ऐनक
	रोटी
	पुस्तक
	महल
	बर्गर
	सूअर
	हाथी
	बतख़
	स्ट्रॉबेरी
	मूंगफली
	उल्लू
	बल्ला
	मेढक
	कांटा
	जिराफ़
	चाकू
	पुस्तिका
	मृग
	पतलून
	कुत्ता
	हाथी
	बिल्ली
	ऊंट
	पनीर
	बिस्कुट
	चेरी
	पोशाक
	कोअला
	मगर
	केक
	लड़की
	लॉरी
	तेंदुआ
	सिंह
	माउस
	दूध
	पांडा
	घोड़ा
	आड़ू
	कुकुरमुत्ता
	पेंगुइन
	\#sauropod
	सेलबोट
	कैंची
	साँप
	घोंघा
	\#schoppen
	जूता
	सूअर
	बेकन
	टैको
	कप
	फ़ोन
	बाघ
	टमाटर
	पाई
	ट्रैक्टर
	अंगूर
	टी-रेक्स
	ज़ेबरा
	बकरा
	बाइक
	व्हेल
	विमान
	पिन
	बादल
	सूरज
	हाथ
	दिल
	कान
	मधुमक्खी
	चम्मच
	जीभ
	भालू
	अंडा
	डोनट
	डॉल्फिन
	पैनकेक
	जहाज़
	\#oktopus
	द्वीप
	}
	#. ja-Hira   hiragana
	#. ja-Kana   katakana
	#. ja-Hrkt   both above
	set ja {
	さる
	あり
	パイナップル
	アップル
	しせん
	め
	くるま
	バナナ
	ツリー
	やま
	ビール
	なし
	プレッツェル
	キャンディ
	めがね
	パン
	ブック
	-じょう
	バーガー
	いのしし
	ぞう
	かも
	イチゴ
	ピーナツ
	ふくろう
	バット
	かわず
	フォーク
	ジラフ
	ナイフ
	さっし
	しか
	ズボン
	いぬ
	ヘッジホッグ
	ねこ
	キャメル
	チーズ
	ビスケット
	チェリー
	ドレス
	コアラ
	ワニ
	ケーキ
	ひよこ
	かもつじどう-しゃ
	\#ヒ
	\#
	マウス
	ミルク
	パンダ
	うま
	もも
	キノコ
	ペンギン
	りゅうきゃく-るい
	ヨット
	はさみ
	へび
	まきがい
	\#schoppen
	くつ
	ぶた
	ベーコン
	タコス
	カップ
	でんわ
	とら
	トマト
	パイ
	トラクター
	ぶどう
	\#T-R
	シマウマ
	じてん-しゃ
	\#
	クジラ
	こうくうき
	ピン
	くも
	たいよう
	\u3066
	こころ
	みみ
	はち
	スプーン
	ぜつ
	クマ
	たまご
	ドーナツ
	イルカ
	パンケーキ
	ふね
	\#Oktopusの
	しま
	}
	#.\##て hand
	#. when . is appended to \u3066, submenu is correct
	#. 
	#. tadjik as ru
	#. tg тадшик
	set tg {
	маймун
	мурча
	ананас
	себ
	чашмони
	чашм
	мошин
	банан
	дарахт
	кӯҳ
	пиво
	нок
	\#pretzel
	конфет
	\#spectacles
	нон
	китоб
	щулф
	Бургер
	\#boar
	фил
	мурцобӣ
	тути\ заминӣ
	арахис
	бум
	муши\ паррон
	щурбощща
	дуроҳа
	заррофа
	корд
	буклет
	оҳу
	шим
	саг
	хорпушт
	гурба
	шутур
	панир
	фатиркулча
	гелос
	либос
	\#koala
	крокодил
	торт
	\#chick
	боркаш
	леопард
	Лео
	муш
	шир
	Панда
	асп
	шафтолу
	занбӯруц
	пингвин
	\#sauropod
	киштӣ
	нущроз
	мор
	тӯщумшуллущ
	\#schoppen
	кафш
	хуг
	хуг
	\#taco
	коса
	телефон
	паланг
	помидор
	пирог
	трактор
	ангур
	т-Рекс
	гӯрхар
	буз
	дучархаи
	наҳанг
	ҳавопаймо
	пайвандак
	абр
	офтоб
	даст
	дил
	гӯш
	занбур
	\#spoon
	забон
	хирс
	тухм
	\#donut
	делфин
	\#pancake
	киштӣ
	\#oktopus
	ҷазира
	}
	#. gujarati ગુજરાતી
	set gu {
	મંકી
	કીડી
	અનેનાસ
	સફરજન
	આંખો
	આંખ
	કાર
	બનાના
	વૃક્ષ
	પર્વત
	બીયર
	પિઅર
	પ્રેટ્ઝેલ
	કેન્ડી
	ચશ્માના
	બ્રેડ
	પુસ્તક
	કિલ્લાના
	બર્ગર
	ભૂંડ
	હાથી
	બતક
	સ્ટ્રોબેરી
	મગફળીના
	ઘુવડ
	બેટ
	દેડકા
	કાંટો
	જિરાફ
	છરી
	પુસ્તિકા
	હરણ
	ટ્રાઉઝર
	કૂતરો
	હેજહોગ
	બિલાડી
	ઊંટ
	પનીર
	બિસ્કિટ
	ચેરી
	ડ્રેસ
	કોઆલા
	મગર
	કેક
	{પક્ષીનું બચ્ચું }
	લૉરી
	ચિત્તો
	લીઓ
	માઉસ
	દૂધ
	પાંડા
	ઘોડો
	પીચ
	મશરૂમ
	પેંગ્વિન
	\#sauropod
	વહાણના
	કાતર
	સાપ
	ગોકળગાય
	\#schoppen
	જૂતા
	ડુક્કર
	બેકન
	ટેકો
	કપ
	ફોન
	વાઘ
	ટમેટા
	પાઇ
	ટ્રેક્ટર
	દ્રાક્ષ
	T-રેક્સ
	ઝેબ્રા
	બકરી
	બાઇક
	વ્હેલ
	વિમાન
	પિન
	વાદળ
	સૂર્ય
	હાથ
	હૃદય
	કાન
	મધમાખી
	સ્પૂન
	જીભ
	રીંછ
	ઇંડા
	મીઠાઈ
	ડોલ્ફીન
	પેનકેક
	જહાજ
	\#oktopus
	ટાપુ
	}
	#. ku kurdî
	set ku {
	meymûn
	gêrîk
	ananas
	sêv
	çavên
	çav
	car
	mûz
	dar
	çîya
	bîra
	hirmê
	pretzel
	candy
	berçavk
	nan
	pirtûk
	kelehê\ de
	burger
	boar
	fîl
	werdek
	tû
	fisteqê\ erdê
	kûnd
	paçmêlk
	beq
	milêvdanî
	giraffe
	kêr
	pirtûkok
	ahû
	şal
	dog
	jîjo
	pisîk
	hoştir
	penêr
	piskûvrît
	gelyaz
	dress
	koala
	tîmseh
	cake
	chick
	qemyûn
	leopar
	leo
	mişk
	şîr
	panda
	hesp
	şiftelî
	kûvarik
	pengûwîn
	sauropod
	bota\ bapaçavî
	meqes
	mar
	şeytanok
	şûşeya\ pitikê
	pêlav
	beraz
	dohn
	taco
	tas
	telefonê
	piling
	bacanê\ sor
	paste
	trektor
	tirî
	t-rex
	zebra
	bizin
	bike
	kuşter
	balafirên
	pin
	ewr
	tav
	dest
	dil
	guhê
	bee
	çavreşa
	ziman
	bear
	hêk
	donut
	masîyê\ ûnis
	pancake
	gemî
	oktopus
	girav
	}
	set ku_lower {a b c ç d e ê f g h i î j k l ł m n ň o p q r ř s ş t u û ü v w x y z}
	#. turkish Türk
	set tr {
	maymun
	karınca
	ananas
	elma
	gözleri
	göz
	araba
	muz
	ağaç
	dağ
	bira
	armut
	pretzel
	şeker
	gözlük
	ekmek
	kitap
	kale
	burger
	domuz
	fil
	ördek
	çilek
	{yer fıstığı}
	baykuş
	yarasa
	kurbağa
	çatal
	zürafa
	bıçak
	kitapçık
	geyik
	pantolon
	köpek
	kirpi
	kedi
	deve
	peynir
	bisküvi
	kiraz
	elbise
	koala
	timsah
	kek
	civciv
	kamyon
	leopar
	{aslan burcu}
	fare
	süt
	panda
	at
	şeftali
	mantar
	penguen
	sauropod
	yelkenli
	makas
	yılan
	salyangoz
	biberon
	ayakkabı
	domuz
	{domuz pastırması}
	taco
	fincan
	telefon
	kaplan
	domates
	turta
	traktör
	üzüm
	t-rex
	zebra
	keçi
	bisiklet
	balina
	uçak
	pim
	bulut
	güneş
	el
	kalp
	kulak
	arı
	kaşık
	dil
	ayı
	yumurta
	{tatlı çörek}
	yunus
	krep
	gemi
	oktopus
	ada
	}
	#. 
	#. malayalam
	#. ml 639-1
	#. mal 639-2
	#. mal 639-3
	#. zhuyin or bopomofo ISO_15924
	#. zh ㄅㄆㄇㄈ
	set zh {
	ㄏㄡˊ\ ㄗˇ
	ㄇㄚˇ\ ㄧˇ
	ㄅㄛ\ ㄌㄨㄛˊ
	ㄆㄧㄥˊ\ ㄍㄨㄛˇ
	ㄧㄢˇ\ ㄐㄧㄥ
	ㄧㄢˇ
	ㄑㄧˋ\ cㄏㄜ
	ㄒㄧㄤ\ ㄐㄧㄠ
	ㄕㄨˋ
	ㄕㄢ
	ㄆㄧˊ\ ㄐㄧㄡˇ
	ㄌㄧˊ
	ㄐㄧㄠ\ ㄧㄢˊ\ ㄐㄩㄢˋ\ ㄅㄧㄥˇ
	ㄊㄤˊ\ ㄍㄨㄛˇ
	ㄧㄢˇ\ ㄐㄧㄥˋ
	ㄇㄧㄢˋ\ ㄅㄠ
	ㄕㄨ
	cㄏㄥˊ\ ㄅㄠˇ
	ㄏㄢˋ\ ㄅㄠˇ
	ㄍㄨㄥ\ ㄓㄨ
	ㄒㄧㄤˋ
	ㄧㄚ
	ㄘㄠˇ\ ㄇㄟˊ
	ㄏㄨㄚ\ ㄕㄥ
	ㄇㄠ\ ㄊㄡˊ\ ㄧㄥ
	ㄅㄧㄢ\ ㄈㄨˊ
	ㄑㄧㄥ\ ㄨㄚ
	cㄏㄚ
	cㄏㄤˊ\ ㄐㄧㄥˇ\ ㄌㄨˋ
	ㄉㄠ
	ㄒㄧㄠˇ\ ㄘㄜˋ\ ㄗˇ
	ㄌㄨˋ
	ㄎㄨˋ\ ㄗˇ
	ㄍㄡˇ
	ㄨㄟˋ
	ㄇㄠ
	ㄌㄨㄛˋ\ ㄊㄨㄛˊ
	ㄋㄞˇ\ ㄌㄠˋ
	ㄅㄧㄥˇ\ ㄍㄢ
	ㄧㄥ\ ㄊㄠˊ
	ㄌㄧㄢˊ\ ㄧ\ ㄑㄩㄣˊ
	ㄎㄠˇ\ ㄌㄚ
	ㄜˋ\ ㄩˊ
	ㄉㄢˋ\ ㄍㄠ
	ㄒㄧㄠˇ\ ㄐㄧ
	ㄏㄨㄛˋ\ cㄏㄜ
	ㄅㄠˋ
	ㄕ\ ㄗˇ\ ㄗㄨㄛˋ
	ㄕㄨˇ\ ㄅㄧㄠ
	ㄋㄧㄡˊ\ ㄋㄞˇ
	ㄒㄩㄥˊ\ ㄇㄠ
	ㄇㄚˇ
	ㄊㄠˊ
	ㄇㄛˊ\ ㄍㄨ
	ㄑㄧˇ\ ㄜˊ
	ㄒㄧ\ ㄐㄧㄠˇ\ ㄌㄟˋ
	ㄈㄢ\ cㄏㄨㄢˊ
	ㄐㄧㄢˇ\ ㄉㄠ
	ㄕㄜˊ
	ㄨㄛ\ ㄋㄧㄡˊ
	\#schㄛ˙pㄆㄣ˙
	ㄒㄧㄝˊ
	ㄓㄨ
	ㄒㄩㄣ\ ㄖㄡˋ
	ㄊㄚˇ\ ㄎㄜ
	ㄅㄟ
	ㄉㄧㄢˋ\ ㄏㄨㄚˋ
	ㄏㄨˇ
	ㄒㄧ\ ㄏㄨㄥˊ\ ㄕˋ
	ㄒㄧㄢˋ\ ㄅㄧㄥˇ
	ㄊㄨㄛ\ ㄌㄚ\ ㄐㄧ
	ㄆㄨˊ\ ㄊㄠˊ
	ㄅㄚˋ\ ㄨㄤˊ\ ㄌㄨㄥˊ
	ㄅㄢ\ ㄇㄚˇ
	ㄕㄢ\ ㄧㄤˊ
	ㄗˋ\ ㄒㄧㄥˊ\ cㄏㄜ
	ㄐㄧㄥ
	ㄈㄟ\ ㄐㄧ
	ㄒㄧㄠ
	ㄩㄣˊ
	ㄊㄞˋ\ ㄧㄤˊ
	ㄕㄡˇ
	ㄒㄧㄣ\ ㄗㄤˋ
	ㄦˇ\ ㄉㄨㄛˇ
	ㄇㄧˋ\ ㄈㄥ
	ㄕㄠˊ\ ㄗˇ
	ㄕㄜˊ\ ㄊㄡˊ
	ㄒㄩㄥˊ
	ㄐㄧ\ ㄉㄢˋ
	ㄧㄡˊ\ ㄓㄚˋ\ ㄑㄩㄢ\ ㄅㄧㄥˇ
	ㄏㄞˇ\ ㄊㄨㄣˊ
	ㄅㄧㄥˇ\ ㄗˇ
	cㄏㄨㄢˊ
	\#Oktㄛ˙ㄆㄨ˙s\ ㄐㄧㄡˇ\ ㄉㄧㄢˋ
	ㄉㄠˇ
	}
		set jp [split $lang -]
		if {[llength $jp]} {
			set lang [lindex $jp 0]
			set langsub [lindex $jp 1]
			set mlang $lang

		}

		#. next two lines required for compatibility with ?
		set hashme ""
		set ::contact_time [clock seconds]

		set k [dict create kin ikinyarwanda uz o‘zbekcha or ଓଡ଼ିଆ pt português es espanol it italiano nl nederlands en english de deutsch fr français ru русский ar العربية mn Монгол fi suomalainen ur اردو fa فارسی hu magyar iw עברי yi ייִדיש mr मराठी hi हिन्दी  ja 日本の tg тадшик gu ગુજરાતી  ku kurdî tr Türk zh ㄅㄆㄇㄈ ps پښتو yo yorubo swa kiswahili zu isizulu pl polskie tam தமிழ் ibo igbo]

		#.foreach iso {de en fr} {}
		foreach iso {kin uz or it pt es ar de en fr fa fi hu nl iw yi ur ru mn mr hi {ja- hrkt} ja-hira ja-kata tg gu ku tr zh ps yo swa zu pl tam ibo} {
			#. globe with meridians
			set text [format $iso_symbol%s $iso]
			#. below becomes pict.
			append text \t $iso
			#. submenu failes
			#.append text \t

			lappend tmp $text
			append hashme $text
			set pnr($text) _lang_$iso

		}

		if {![string length $lang]} {
			set lang [string range $::env(LANG) 0 1]
		}

		my direction $lang
		my hide_controls
		my geometry
		my show_controls
		
		#. check whether to use render_arabic in picture
		#. set render_arascr [expr {!$ltr && [lsearch {ar fa ur ps} $lang]>-1 ? 1 : 0}]
		set render_arascr [expr {!$ltr && [string equal $lang ar] ? 1 : 0}]


			
		set max [llength $uni]
		set i 0

		#.from now on
		
		set lish unknown
		set lish [dict get $k $lang]

		borg toast $lish
		if {[lsearch {bo} $lang]>-1} {


			borg toast "try bopo"
			set lang {}
			set num [borg speak $lish]

		} else {
			set num [borg speak $lish $lang]

		}

		borg toast "$lish $lang: $num"
		
		#. num should be > 1000
		if {$num < 0} {
			if {[lsearch {ar fa ur ps} $lang]>-1} {
				if {[string equal $lang ar]} {
					set res [borg speak [lindex $ar 0] ar]
					if {$res<0} {
						set res [borg speak [lindex $ar 0] ur]
						if {$res > 0} {
							set mlang ur
						}
					}

				} else {
					#. speak fa as ur
					set mlang ur
					borg speak $lish ur
				}
			} else {

				if {[lsearch {iw yi} $lang]>-1} {
					#. keep
					set mlang $lang
				} elseif {[lsearch {yo zu ibo} $lang]>-1} {

					set mlang es
					#.set mlang it
				} else {

				}

				borg toast [borg speak $lish $mlang]
			}


			if {![string equal $lang $mlang]} {


				#. Android 5.1 , not 4.1
				borg toast "$lang as $mlang" 
			}
		}

		if {$ltr} {
			set pos1 $lang
			append pos1 \u300b
		} else {
			set pos1 \u300a
			append pos1 $lang
		}
		$delta configure -text $pos1

		set text [format "$iso_symbol%s %s" $mlang $lish]
		if {$ltr} {
			$ltx configure -text $text
		} else {
			$ltx configure -text [string reverse $text]
		}

		my kwiz_init


		array unset ftsi
		array set ftsi {}

		#. ja-hrkt both hiragana and katakana

		if {[string equal ja $lang]} {

			set filter 1
			if {[string equal $langsub hira]} {
				#. ja-kata
				#. 30fc prolonged sound sign
				set pat {^[\u3040-\u309f \u30fc]}
			} elseif {[string equal $langsub kata]} {
				#. hiragana only
				#. ja-hira
				set pat {^[\u30a0-\u30ff]}
			} else {
				#. " hrkt"
				set filter 0
			}
			#.foreach n [set [set lang]] u $uni {}
			foreach x $ja u $uni {
				if {$filter && ![regexp $pat $x]} {
					continue
				}

				append hashme $x
				append x \t $u
				lappend tmp $x
				incr i
				if {$i>$max} {
					#. check for multiple words in line
					error "mismatch words/symbols"
				}
			}
		} else {
			foreach n [set [set lang]] u $uni {
				if {![regexp {^#} $n]} {



					set x [string tolower $n]
					append hashme $n
					append x \t $u
					lappend tmp $x

				}
				incr i
				if {$i>$max} {
					#. check for multiple words in line
					error "mismatch words/symbols"
				}
			}
			#. create e.g. en_lower
			set range $lang
			append range _lower
			if {[info exists $range]} {

				foreach low [set $range] {
					set upper [string toupper $low]

					#. tiny space
					if {[string equal $low $upper]} {
						lappend tmp "\u2009$low \t $low "
					} else {
						lappend tmp "\u2009$low \t $upper "
					}
				}
			}
		}

		catch {
			array unset ftsi
			array set ftsi {}
		}



		set tmp
	}
	method sw {{x {}}} {
		puts "sw:x=<$x>"
		set res [list]
		if {[catch {set n $yy([json_string [set x]])} err]} {
puts "err=<$err>"
			if {[array size yy] == 1} {
				set yy() [array names yy]
				my sw
			}
			return {}
		}
		set r {}
		set r [split $n \r]
		set r [lindex $r 1]
		set lines [split $r \n]
		foreach x $lines {
			if {[string length $x]} {
				if {[regexp \v $x]} {
					foreach {fin cont} [split $x \v] break
					lappend res [list $fin $cont]
				} else {
				}
			} else {
			}
		}
		return $res
	}
	method force_initials {sorted_list} {
		puts "force_initials:start"
		#. insert initial to sorted_list, when initial's count is 1
		upvar $sorted_list ll
		set err nix
		catch {array unset force_i} err

		array set force_i {}

		foreach n $ll {

			set ia [string range $n 0 0]
			incr force_i($ia)
		}
		set tmp {}
		if {1} {
			#. moves symbols on startscreen to submenu



			#. singualar has char, that must not appear at second position
			#. of any element

			foreach {k v} [array get force_i] {

				if {$v==1} {
					set m $k

					#. this shift single startwords to next level
					append m "."
					set add $m
					append add \t $k
					lappend tmp $add

					array set pnr {$k $k}
				}
			}

		}



		return [lsort -ascii [concat $tmp $ll]]
	}
	method no_explode {tup_list} {
		array set pnr {}
		set ph {}
		foreach item $tup_list {
			#.puts "no_explode: item=<$item>"
			set z [split $item \t]
			foreach {name nr} $z break
			array set pnr [list $name $nr]
			lappend ph $name
		}
		set fh [open pnr.txt w]
		puts $fh [array get pnr]
		close $fh
		#.vwait ::kk
		return $ph
	}
	method column_wise {inlist} {

		set n [llength $inlist]
		set per_column [my oddlist $n]
		set k 0
		#. first index will be 0
		set sum -1
		set res [list]
		foreach z $per_column {
			set sub [list]
			while {$k<$z} {
				lappend sub [incr sum]
				incr k
			}
			set k 0
			lappend res $sub
		}
		set res
	}
	method oddlist {{z 23}} {
		variable display_columns
		set display_columns 1

		if {![catch {set res $odds($portrait,$z)} err]} {

			return $res
		} else {
			#. initialize array
			array set odds {
			0,2 {1 1}
			0,3 {2 1} 
			0,4 {2 2}
			0,5 {3 2}
			0,6 {3 3}
			0,7 {4 3}
			0,8 {3 2 3}
			0,9 {3 3 3}
			1,3 {3}
			}
		}
		#. file odd.tcl
		if {$portrait} {


			set percol 8


			set minpercol 6 






		} else {

			#.set percol 6
			set percol 3
			#.set minpercol 5
			set minpercol 3
		}

		if {$z>$percol} {

			set n [expr {int($z**.5)}]

			if {$portrait} {
				set n [expr {int($n*.8)}]
			} else {
				set n [expr {int($n*1.3)}]
			}
			set display_columns $n

	#. n is number of columns
			set rows_per_column [expr {$z/$n}]
			set prod [expr {$n*$rows_per_column}]
			set rest [expr {$z-$prod}]
	#. below looks strange with 2 columns z=24 (3x8)->(2x12) 4x6
	#. <ur> startscreen
	#.		if {!$rest && $n>2} {
	#.			#. avoid view of a table
	#.			incr n -1
	#.			set rows_per_column [expr {$z/$n}]
	#.			set prod [expr {$n*$rows_per_column}]
	#.			set rest [expr {$z-$prod}]
	#.		}
			if {!$rest && $n>2} {
				#. avoid view of a table
				incr n
				set rows_per_column [expr {$z/$n}]
				set prod [expr {$n*$rows_per_column}]
				set rest [expr {$z-$prod}]
			}
			set res [list {*}[string repeat \ $rows_per_column $n]]


			if {$rest} {
				#. get different row counts in first columns from left to right
				#. use alternating rows in subsequent columns when possible, second column always unchanged
				set max [expr {$rows_per_column+1}]
				set pos 0
				lset res $pos $max
				incr rest -1
				#. number of remaining columns
				incr n -2
				set add 1
				incr pos
				while {$rest} {
					#. leave second column as is
					incr pos $add
					#. 200131 bug: list index out of range after rot. mr - lang portrait to landscape

					lset res $pos $max
					#. skip next column when possible
					set add [expr {$rest<$n ? 2 : 1}]
					incr n -$add
					incr rest -1
				}
			}
		} else {
			set res [list $z]
		}
		set odds($portrait,$z) $res 
		return $res
	}
	method xx_escape i {
		return [string map {* \\* ? \\? \\ \\\\ \[ \\\[ \] \\\]} $i]
	}
	method nada_mem {prefix fontname} {
		if {1} {
			#.puts "nada_mem:prefix=<$prefix>,fontname=$fontname"

			set Rh [winfo screenheight $R]
			set h [winfo rooty $header]
			puts "h=$h"
			set wh [winfo height $header]
			puts "+++wh=$wh"
			if {$wh == 1} {
				pack $ltx -side top -fill x
				pack $header -side top -fill x -expand 1 -anchor n
				update idletasks
				set wh [winfo height $header]
				incr wh [winfo height $ltx]
				puts "+++wh=$wh"
			}

			incr h $wh

			puts "h=$h"

			set h [expr {$Rh - $h}]
			puts "Rh=$Rh, h=$h"

		}
		
		set w [winfo screenwidth .k]

puts "tau geometry set to $w x $h"

		#. bg should not be visible
		#. depends whether l_suffix expands to row length
		$tau configure -width $w -height $h -borderwidth 0 -bg {}
		$tau configure -width $w -height $h -borderwidth 0 -bg {}
		#. vkb shifts down: pack propagate .k.f 0
		pack propagate $tau 0


puts "vkb geometry set to $w x $h"
		#. bg affects vertical borders
		$vkb configure -width $w -height $h -bg {}
		#. default required: pack propagate $vkb 1
		set vkb_width $w
		set vkb_height $h
		#. avoid flicker on resize font
		pack forget $vkb $tau $header


		my ada
		return
	}
	method ada {} {
		variable first_time
		variable display_columns
puts "ada:start"


		
puts "display_columns=$display_columns"	
		set minsize [expr {-[winfo width $R]/40}]
		set minsize [expr {int($minsize/$display_columns)}]
		set maxsize [expr {-[winfo width $R]/2}]
		set maxsize [expr {int($maxsize/$display_columns)}]
puts "maxsize=$maxsize   minsize=$minsize"
		set step [expr {($maxsize-$minsize)/3}]
		if {$step>=0} {
			set step -4
		}
puts "step=$step"
		set xy_width [list]
		set xy_height [list]
		for {set n $minsize} {1} {incr n $step} {
			if {[my check_rotation_stop]} {
				return
			}
puts "n=$n"
			my resize_font $n
			update idletasks
			set actwidth [winfo reqwidth $vkb]
			lappend xy_width $actwidth
			lappend xy_width $n

			set actheight [winfo reqheight $vkb]
			lappend xy_height $actheight
			lappend xy_height $n

			if {$actwidth >=$vkb_width || $actheight >= $vkb_height} {
				set size_w [math::interpolate::interp-linear $xy_width $vkb_width]
				set size_h [math::interpolate::interp-linear $xy_height $vkb_height]
				if {$size_w < $size_h} {
					set size_w $size_h	
				}
				if {!$istouch} {
					#. obvious difference Androwish/undroid
					set div [expr {$first_time ? 4 : 2}]
					set first_time 0
					set size_w [expr {$size_w/$div}]
				}
				my resize_font [expr {int($size_w)}]
				#. works, but flickers

				after 400 [list [self] repack]
				return
			}
		}
	}
	method resize_font {size} {
		#.puts "resize_font: size=$size, current=$size_current"
		if {$size != $size_current} {

			#.k.ltx configure -text $size
			font configure $dfont -size $size
			set size_current $size
			if {[lsearch {ar fa ja ur} $mlang] > -1} {
				set picsize [expr {int(1.3*$size)}]
			} else {
				set picsize [expr {int(2.*$size)}]
			}
			font configure $picfont -size $picsize
		}

	}
	method frameclick {final {completion {}}} {

		$input delete 0 end
		set cc [string range $completion 0 0]
		if {$final} {
			catch {$input insert end $cc}

			#. important
			after idle [list [self] append_suffix $final $completion]
			#.my append_suffix $final $completion
		} else {
			#. no case toggle required (recursive=1)
			after idle [list [self] newchar $cc 0 1]
			#. bad my newchar $cc 1
			#. freeze

		}

		return
	}
	method bc_row {parent eff} {
puts "bc_row:$parent, $eff"
		set bgs {white #ecf }
		set bgs {white blue}
		set bgs {white white}
		set e [list]
		lappend e $eff
		lappend e {}
		set newlabels [list]
		foreach x $e {
			if {!$ltr} {
				set x [string reverse $x]
			}


			set bg [lindex $bgs [expr {$bcidx%2}]]
puts "bc_row:bg=$bg"
			if {[string length $x]} {

				set el [label $parent.$bcidx -text $x -bg $bg]
				#. to small
				#. bind $el <Button-1> [list [self] backstep]
			} else {
				#. separator between completions
				set el [frame $parent.$bcidx -width 2 -bg black]
			}
			lappend newlabels $el
			incr bcidx
		}
		puts "bcrow:bcidx=$bcidx"




		#. fill y is responsible to show  vertical separator in multiline mode of suffix




		pack forget $prefix_frame
		pack {*}$newlabels -side $lr -anchor $an -fill y
pack $prefix_frame -side $lr -anchor $an -fill both -expand 1 -before $input



		#



		puts bc_row:end
	}
	method setwraplength {} {

		set z [winfo width $suffix]

		$suffix configure -wraplength $z 
		set h [winfo reqheight $suffix]

		#. set x [winfo height .h]
		#. if {$h < $x} {set h $x}
		$suffix configure -height -$h

		return $z
	}
	method adapt_headerfont {} {

		if {[info exists aftersuffix]} {
			if {1} {
				set cw [my setwraplength]
				if {!$ltr} {
					set text [$suffix cget -text]
					set tlen [string length $text]
					set f [$suffix cget -font]
					set w_h 0
					for {set i 1} {$i<$tlen} {incr i} {
						set sub [string range $text 0 $i]
						set w [font measure $f $sub]
						if {$w >= $cw} {
							break
						}
						set w_h $w
					}
					$suffix configure -wraplength $w_h
					#. width w in pixel corresponds to string lenght i
					set tmp [textutil::splitn $text $i]


					set rev [list]
					foreach n $tmp {

						lappend rev [string reverse $n]
					}

					set text [join $rev \n]

					$suffix configure -text $text
				}










			} else {
				#. only size decreasing
				if {![winfo ismapped $aftersuffix]} {
					#. on reload is not packed
					catch {pack $aftersuffix}
					set info [pack info $aftersuffix]
					if {$portrait && [regexp ^- $info]} {
						#. negative sizes used
						set k0 [$suffix cget -text]

						append k1 [regsub -all { } $k0 \n]
						$suffix configure -text $k1
						#. get widgets top aligned






						#. else "window .. is'nt packed"
					}
				}
			}
		} else {

		}

	}
	method kwiz_next {} {
		#. retrieve mode is ended when last item is matched
		#. or skipped by touching the top row

		#. show next to find
		set kwizact [lindex $kwizlist end]

		set kwizlist [lrange $kwizlist 0 end-1]

		if {[string length $kwizact]} {
			incr kwizcount
			set info $kwizcount/$kwiztotal
			append info " " $kwizact

			$ltx configure -text $info -bg white
			my p_speak $kwizact
		}

	}
	method kwiz_check {input} {
		#. check if input matches picture in top row
		#. audible feedback
		#. repeats item to find on mismatch

		if {[string equal $input $kwizact]} {

			#. higher pitch
			borg speak [my abbrev $mod($input)] $mlang 1.2 1.
			if {$kwizcount<$kwiztotal} {
				my kwiz_next
			} else {
				$ltx configure -text [format %d/%d [expr {$kwiztotal-$kwizskipped}] $kwiztotal]
				my kwiz_init
			}
		} else {


			after idle [list [self] mis_speak $input]

		}

	}
	method kwiz_collect {input} {
		#. collect items to retrieve later
		#. retrieve is started by touching the item list in top row

		set n [lsearch $kwiz_visited $input]
		if {$n<0} {
			lappend kwiz_visited $input

			$ltx configure -text [llength $kwiz_visited]:\ [join $kwiz_visited "."]
		}
	}
	method pkwiz {{skip 0}} {


		if {$skip} {
			incr kwizskipped
		} else {
			borg stopspeak
			if {![llength $kwizlist]} {
				if {[llength $kwiz_visited]} {

					set kwizlist [shuffle9 $kwiz_visited]
					set kwiz_visited {}
					set kwizcount 0
					set kwiztotal [llength $kwizlist]
					my hide_pics
					if {$istouch} {
						bind $ltx <Button-1> [list [self] pkwiz 1]
					} else {
						bind $ltx <Button-1> [list [self] pkwiz 1]
					}
				}
			}
		}
		if {$kwizcount<$kwiztotal} {
			my kwiz_next
		} else {
			$ltx configure -text [format %d/%d [expr {$kwiztotal-$kwizskipped}] $kwiztotal]
			my kwiz_init
		}

		return
	}
	method show_pics {} {

		my tfon show_pics

	}
	method check_prefix {x} {

		set current [my current_prefix]
		append current $x

		set required ""
		catch {set required $mod($kwizact)}

		set pos [string first $current $required]

		if {$pos != 0} {
			after idle [list borg speak $current $mlang .6 .7]
			after idle [list [self] p_speak $kwizact]
			return 1
		}
		return 0
	}
	method mis_speak {u} {

		#. u might be "piece of cheese", while y is "cheese"

		if {[catch {set y $mod($u)}]} {
			#.set auto_back 1
			set y [my current_prefix]
		} else {
			#.set auto_back 0
		}

		borg speak $y $mlang .6 .7
		my p_speak $kwizact

		#.return $auto_back
	}
	method abbrev {x} {
		#. suppress underscore prefix
		regsub {^.*_} $x ""
	}
	method p_speak {u} {

		set x [my var mod($u)]

		borg speak [my abbrev $x] $mlang
	}
	method hide_pics {} {

		if {[llength $pics]} {
			foreach p $pics {

				pack forget $p
			}
		}

	}
	method tele {{lang fr}} {
		puts "tele:lang=$lang"
		set skip_tx 0
		if {[namespace exists ::do]} {
			namespace delete ::do
		}
		namespace eval ::do {}
		set gg -1
		array set mod {}
#.ok delfin\u2007\U1f42c\ \U1f42c
#.
#.bagel\u2007\U1f96f\ \U1f96f
#.dachs\u2007\U1f9a1\ \U1f9a1
#.pfau\u2007\U1f99a\ \U1f99a
#.bein\u2007\U1f9b5\ \U1f9b5
#.fuss\u2007\U1f9b6\ \U1f9b6
#.papagei\u2007\U1f99c\ \U1f99c
#.besen\u2007\U1f9f9\ \U1f9f9
#.korb\u2007\U1f9fa\ \U1f9fa
#.schwamm\u2007\U1f9fd\ \U1f9fd
#.schwan\u2007\U0001f9a2\ \U0001f9a2
#.känguruh\u2007\U1f998\ \U1f998
#.apfel\u2007\U1f34f\ \U1f34f
		#. at least 2 different first characters are required
		#. if a gcp exists of all elements, no element yy() will exist! -> algorithm misconstruction
		#. see catch in proc sw
		array unset odds
		array set odds {}
		set xx [my mylang $lang] 
		puts "tele:xx [llength $xx]"
		set tmp [my force_initials xx]

		#. set tmp [my mylang $lang] 
		$ltx configure -text $lang

		#. non breaking space avoids tilde
		#. table of numerals
		#. https://en.wikipedia.org/wiki/Hindu%E2%80%93Arabic_numeral_system#Glyph_comparison
		if {[string equal $mlang hi]} {
			#. devanagari
			for {set i 2406} {$i<2416} {incr i} {
				set u [subst -nocommands -novariables [format \\u%04x $i]]
				lappend tmp \u2116\ $u\t$u
			}
		} elseif {[string equal $mlang gu]} {
			for {set i 0x0ae6} {$i< 0x0af0} {incr i} {
				set u [subst -nocommands -novariables [format \\u%04x $i]]
				lappend tmp \u2116\ $u\t$u
			}
		} elseif {[string equal $mlang mr]} {

			for {set i 0x0966} {$i< 0x0970} {incr i} {
				set u [subst -nocommands -novariables [format \\u%04x $i]]
				lappend tmp \u2116\ $u\t$u
			}
		} elseif {[string equal $mlang tam]} {

			for {set i 0x0be6} {$i< 0x0bf0} {incr i} {
				set u [subst -nocommands -novariables [format \\u%04x $i]]
				lappend tmp \u2116\ $u\t$u
			}
			#. next 3 are 10, 100, 1000
		} elseif {[string equal $mlang zh] || [regexp {^ja} $mlang]} {
			#. Chinese_Numerals (wp)
			#. 12295 0
			#. 19968 1
			#. 20108 2
			#. 19977 3
			#. 21313 10
			set k 0
			foreach i {12295 19968 20108 19977 22235 20116 20845 19971 20843 20061 21313} {
				set u [subst -nocommands -novariables [format \\U%06x $i]]

				lappend tmp "\u2116 _$u\t$u"
				incr k
			}
		}
		foreach x {{ 0} { 1} { 2} { 3} { 4} { 5} { 6} { 7} { 8} { 9} 10 11 12 13 14 15 16 17 18 19 20 30 40 50 60 70 80 90 100 1000} {

			lappend tmp \u2116$x\t[string trim $x]
			#.if {!$ltr} {
			#.	lappend tmp \u00a0[string reverse $x]\t$x
			#.} else {
			#.	lappend tmp \u00a0$x\t$x
			#.}
		}
		if {1} {
			#. this is injected when language is changed

			lappend tmp ${iso_symbol}_AndroWish\t//androwish.org
			lappend tmp ${iso_symbol}_Andra\u00a0Megawidget\t//tcl.tk
			lappend tmp ${iso_symbol}_Deltadatentechnik\u00a0GmbH\t//deltadatentechnik.de
			lappend tmp ${iso_symbol}_download\t//taipu.de/andra

			lappend tmp ${iso_symbol}\u2623_biological\ hazard\t\u2623

			#. alien
			#. lappend tmp ${iso_symbol}\u2623biohazard\ agent\t\u2623\U1f47d
			#. lappend tmp ${iso_symbol}\u2623biohazard\ agent\t\u2623RNA
			lappend tmp ${iso_symbol}\u2623_biohazard\ agent\t\u2623\U1f451
			#. dna
			#. lappend tmp${iso_symbol}\u2623biohazard\ agent\t\U1f9ec
			lappend tmp ${iso_symbol}\u2623_biohazard\ host\t\u2623\U1f912
			lappend tmp ${iso_symbol}\u2623_biohazard\ origin\t\u2623\U1f987
			lappend tmp ${iso_symbol}\u2623_biohazard\ transmission\t\u2623\U1f927
		}

		set validcount [llength $tmp]
		if {$validcount == 0} {
			tk_messageBox -message "no data found"
			exit
		} elseif {$validcount == 1} {
			foreach {name contact_id} [lindex $tmp 0] break
			catch {array unset yy}
			array set yy [list {} \r\n1\v$name]
			catch {array unset pnr}
			array set pnr [list $name $contact_id]

		} elseif {$validcount==2} {
			catch {array unset yy}
			foreach {first c0} [lindex $tmp 0] break
			foreach {second c1} [lindex $tmp 1] break
			set prefix [gcp $first $second]
			set k [string length $prefix]
			if {$k} {
				set yy() \r\n0\v$prefix
				set yy($prefix) \r\n1\v[string range $first $k end]\n1\v[string range $second $k end]
			} else {
				set yy() \r\n1\v$first\n1\v$second
			}
			catch {array unset pnr}
			array set pnr [list $first $c0 $second $c1]

		} else {
			#. needs rework, as files are not language specific

			set fname pollux.txt
			if {[file exists pollux.tx]} {
				set contact_time [file mtime pollux.tx]

				if {![info exists hashme]} {
					#. see proc mylang
					set hashme "dummy"
				}
				set myhash [::md5::md5 $hashme]

				if {[file exists pollux.md5]} {
					set fh [open pollux.md5 r]
					fconfigure $fh -encoding binary
					set digest [read $fh]
					close $fh
					set skip_tx [string equal $digest $myhash]

				}
			}

			if {!$skip_tx} {
				#. remember refresh-time
				set contact_time [clock seconds]

				file delete crit.txt

				#. write input file for "sorted_file2tx"
				set fh [open $fname w]
				set prev ""
				set line_prev ""
				set skip 0
				#.foreach line [lsort -unique [explode $tmp]] {}
				foreach line [lsort -unique [my no_explode $tmp]] {

					if {[catch {set current $pnr($line)} err]} {

						set current ""
					} else {
					}

					if {![string equal $current $prev]} {
						#. smallest item at index 0 is written
						puts $fh $line
					} else {
						if {![string length [gcp $line $line_prev]]} {
							puts $fh $line
						} else {
							incr skip
						}
					}
					set prev $current
					set line_prev $line
				}
				seek $fh -1 end
				close $fh
				#.puts "before sorted_file2tx: fname=$fname"
				sorted_file2tx $fname
				#.puts "after sorted_file2tx"
				#
				set fh [open dchar.txt w]

				puts $fh [array get ::dchar]
				close $fh

				#
				set myhash [::md5::md5 $hashme]
				set fh [open pollux.md5 w]
				fconfigure $fh -encoding binary
				close $fh
			}
			if {![file exists crit.txt]}  {
				if {[file exists dchar.txt]} {

					set fh [open dchar.txt r]
					array set dchar [read $fh]
					close $fh
					my uca_trans [array names dchar]
				} else {
					catch {file delete pollux.tx}
					my tele
				}
			} else {
				set fh [open crit.txt r]
				array set uca [read $fh]
				close $fh
			}
			set fh [open pnr.txt r]
			array set pnr [read $fh]
			close $fh

			set fh [open pollux.tx r]
			catch {array unset yy}
			array set yy [read $fh]
			close $fh

		}

		puts tele:end
	}
	method uca_response {charlist uncharlist token} {

		set status [http::status $token]

		if {[string equal ok $status]} {
			set all [http::code $token]
			lassign $all p c s
			if {$c==200} {
				binary scan [http::data $token] c* jj
				if {[llength $jj] > 255} {
					binary scan [http::data $token] s* jj
				}
				set dd_max 0
				foreach cc $charlist dd $jj {
					array set uca [list $cc $dd]
					if {$dd>$dd_max} {
						set dd_max $dd
					}
				}
				foreach cc $uncharlist {
					incr dd_max
					array set uca [list $cc $dd_max]
				}
				array set uca {{} -1}

				set fh [open crit.txt w]
				puts $fh [array get uca]
				close $fh
				#. changes appear on user actions when view changes
				catch {
				array unset ftsi 1,
				array unset ftsi 0,
				}
			}
		} else {
			borg toast "request: $status"
		}
		http::cleanup $token

	}
	method uca_trans charlist {

		set sendlist [list]
		set uncharlist [list]
		foreach cc $charlist {
			if {[scan $cc %c i]} {
				if {$i>=65536} {
					lappend uncharlist $cc
				} else {
					lappend sendlist $cc
				}
			}
		}


		set url http://taipu.de/bincoll
		#.set url https://taipu.de/bincoll
		#.http::register https 443 [list ::tls::socket -autoservername true]
		#. -require -cafile ?
		if {[catch {set tok [http::geturl $url -timeout 600 -command [list [self] uca_response $sendlist $uncharlist] -query [http::formatQuery uca [join $sendlist ""] l $mlang] ]} err]} {
			borg toast "no network: using dictionary sort order"
			#.puts "err=$err"
			set jj [lsort -dictionary $charlist]
			foreach c $charlist {
				set idx [lsearch -exact $jj $c]
				array set uca [list $c $idx]
			}
			array set uca {{} -1}
			if {[file exists crit.txt]} {
				file delete crit.txt
			}
		}

		return
	}
	method collcomp {a b} {
		set a [string map {* \\* ? \\? \\ \\\\ \[ \\\[ \] \\\]} $a]
		set b [string map {* \\* ? \\? \\ \\\\ \[ \\\[ \] \\\]} $b]
		#.puts "collcomp:$a,$b"
		foreach  {{} x} [array get uca $a] break
		#.puts x=$x
		foreach  {{} y} [array get uca $b] break
		#.puts y=$y
		return [expr {$x < $y ? -1 : $x==$y ? 0 : 1}]
	}
	method repack {} {
		puts "repack:start"
		pack $header -side top -fill x -expand 1 -anchor n
		pack $tau -side top -fill both -expand 1
		#. evil pack propagate $tau 1
		pack $vkb -side top -fill both -expand 1 -anchor n

		if {[string equal withdrawn [wm state .k]]} {
			wm deiconify .k
		}
		focus $input


		set cwid [after 500 [list [self] check_rotation [winfo width .k]]]
	}
	method check_rotation {w} {
		variable cwid
		set act [winfo width .k]
		if {$act != $w} {
			set first_time 0

			pack forget $vkb $tau $header
			#. causes blackscreen
			#. wm withdraw .k
			my tfon rotated

		} else {
			set cwid [after 900 [list [self] check_rotation $act]]
		}
	}
	method check_rotation_stop {} {
		variable cwid
		if {[string length $cwid]} {
puts "tfon: after cancel cwid=$cwid"
			return 1


		}
		return 0
	}
}





if {$android} {
	wm attributes .k -fullscreen 1
	bind .k <Control-Key-BackSpace> {borg withdraw}
} else {
	proc lan {} {
		wm geometry .k 700x300+0+0

	}
	proc por {} {
		wm geometry .k 300x700+0+0
	}

	lan
}
raise .k .
set ::nn [::andra new $k]

bind .k <<SensorUpdate>> {rota %x}
proc rota {x} {
	puts rota:x=$x
	parray [$x get]
}

if {0} {
	pack [entry .e]
	pack [button .b -text nix]
	raise .
	proc two {a k} {

		if {[string length $a]} {
			.e selection range 0 end
		} else {
			if {$k==43} {
				#. tab
				focus .b
			} elseif {$k==224} {
				#. shift-tab
				focus .b
			}
		}
	}
	bind .e <Any-Key> {two %A %k}
}
#. vi:noexpandtab:ts=4:sw=4:notx
